#ifndef lint
static char const 
yyrcsid[] = "$FreeBSD: src/usr.bin/yacc/skeleton.c,v 1.28.2.1 2001/07/19 05:46:39 peter Exp $";
#endif
#include <stdlib.h>
#define YYBYACC 1
#define YYMAJOR 1
#define YYMINOR 9
#define YYLEX yylex()
#define YYEMPTY -1
#define yyclearin (yychar=(YYEMPTY))
#define yyerrok (yyerrflag=0)
#define YYRECOVERING() (yyerrflag!=0)
#if defined(__cplusplus) || __STDC__
static int yygrowstack(void);
#else
static int yygrowstack();
#endif
#define YYPREFIX "yy"
#line 2 "ns_parser.y"
#if !defined(lint) && !defined(SABER)
static char rcsid[] = "$Id: ns_parser.y,v 8.81.8.1 2003/06/02 09:56:35 marka Exp $";
#endif /* not lint */

/*
 * Copyright (c) 1996-2000 by Internet Software Consortium.
 *
 * Permission to use, copy, modify, and distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND INTERNET SOFTWARE CONSORTIUM DISCLAIMS
 * ALL WARRANTIES WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL INTERNET SOFTWARE
 * CONSORTIUM BE LIABLE FOR ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL
 * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR
 * PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 * SOFTWARE.
 */

/* Global C stuff goes here. */

#include "port_before.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <sys/un.h>

#include <netinet/in.h>
#include <arpa/nameser.h>
#include <arpa/inet.h>

#include <ctype.h>
#include <limits.h>
#include <resolv.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>
#include <time.h>

#include <isc/eventlib.h>
#include <isc/logging.h>

#include <isc/dst.h>

#include "port_after.h"

#include "named.h"
#include "ns_parseutil.h"
#include "ns_lexer.h"

#define SYM_ZONE	0x010000
#define SYM_SERVER	0x020000
#define SYM_KEY		0x030000
#define SYM_ACL		0x040000
#define SYM_CHANNEL	0x050000
#define SYM_PORT	0x060000

#define SYMBOL_TABLE_SIZE 29989		/* should always be prime */
static symbol_table symtab;

#define AUTH_TABLE_SIZE 397		/* should always be prime */
static symbol_table authtab = NULL;
static symbol_table channeltab = NULL;

static zone_config current_zone;
static int should_install;

static options current_options;
static int seen_options;
static int logged_options_error;

static controls current_controls;

static int seen_topology;

static server_config current_server;
static int seen_server;

static char *current_algorithm;
static char *current_secret;

static log_config current_logging;
static int current_category;
static int chan_type;
static int chan_level;
static u_int chan_flags;
static int chan_facility;
static char *chan_name;
static int chan_versions;
static u_long chan_max_size;

static log_channel lookup_channel(char *);
static void define_channel(const char *, log_channel);
static char *canonical_name(char *);

int yyparse();
	
#line 105 "ns_parser.y"
typedef union {
	char *			cp;
	int			s_int;
	long			num;
	u_long			ul_int;
	u_int16_t		us_int;
	struct in_addr		ip_addr;
	ip_match_element	ime;
	ip_match_list		iml;
	rrset_order_list	rol;
	rrset_order_element	roe;
	struct dst_key *	keyi;
	enum axfr_format	axfr_fmt;
} YYSTYPE;
#line 138 "y.tab.c"
#define YYERRCODE 256
#define L_EOS 257
#define L_IPADDR 258
#define L_NUMBER 259
#define L_STRING 260
#define L_QSTRING 261
#define L_END_INCLUDE 262
#define T_INCLUDE 263
#define T_OPTIONS 264
#define T_DIRECTORY 265
#define T_PIDFILE 266
#define T_NAMED_XFER 267
#define T_DUMP_FILE 268
#define T_STATS_FILE 269
#define T_MEMSTATS_FILE 270
#define T_FAKE_IQUERY 271
#define T_RECURSION 272
#define T_FETCH_GLUE 273
#define T_HITCOUNT 274
#define T_PREFERRED_GLUE 275
#define T_QUERY_SOURCE 276
#define T_LISTEN_ON 277
#define T_PORT 278
#define T_ADDRESS 279
#define T_RRSET_ORDER 280
#define T_ORDER 281
#define T_NAME 282
#define T_CLASS 283
#define T_CONTROLS 284
#define T_INET 285
#define T_UNIX 286
#define T_PERM 287
#define T_OWNER 288
#define T_GROUP 289
#define T_ALLOW 290
#define T_DATASIZE 291
#define T_STACKSIZE 292
#define T_CORESIZE 293
#define T_DEFAULT 294
#define T_UNLIMITED 295
#define T_FILES 296
#define T_VERSION 297
#define T_HOSTNAME 298
#define T_HOSTSTATS 299
#define T_HOSTSTATSMAX 300
#define T_DEALLOC_ON_EXIT 301
#define T_TRANSFERS_IN 302
#define T_TRANSFERS_OUT 303
#define T_TRANSFERS_PER_NS 304
#define T_TRANSFER_FORMAT 305
#define T_MAX_TRANSFER_TIME_IN 306
#define T_SERIAL_QUERIES 307
#define T_ONE_ANSWER 308
#define T_MANY_ANSWERS 309
#define T_NOTIFY 310
#define T_EXPLICIT 311
#define T_NOTIFY_INITIAL 312
#define T_AUTH_NXDOMAIN 313
#define T_MULTIPLE_CNAMES 314
#define T_USE_IXFR 315
#define T_MAINTAIN_IXFR_BASE 316
#define T_CLEAN_INTERVAL 317
#define T_INTERFACE_INTERVAL 318
#define T_STATS_INTERVAL 319
#define T_MAX_LOG_SIZE_IXFR 320
#define T_HEARTBEAT 321
#define T_USE_ID_POOL 322
#define T_MAX_NCACHE_TTL 323
#define T_HAS_OLD_CLIENTS 324
#define T_RFC2308_TYPE1 325
#define T_LAME_TTL 326
#define T_MIN_ROOTS 327
#define T_TREAT_CR_AS_SPACE 328
#define T_EDNS_UDP_SIZE 329
#define T_LOGGING 330
#define T_CATEGORY 331
#define T_CHANNEL 332
#define T_SEVERITY 333
#define T_DYNAMIC 334
#define T_FILE 335
#define T_VERSIONS 336
#define T_SIZE 337
#define T_SYSLOG 338
#define T_DEBUG 339
#define T_NULL_OUTPUT 340
#define T_PRINT_TIME 341
#define T_PRINT_CATEGORY 342
#define T_PRINT_SEVERITY 343
#define T_SORTLIST 344
#define T_TOPOLOGY 345
#define T_SERVER 346
#define T_LONG_AXFR 347
#define T_BOGUS 348
#define T_TRANSFERS 349
#define T_KEYS 350
#define T_SUPPORT_IXFR 351
#define T_EDNS 352
#define T_ZONE 353
#define T_IN 354
#define T_CHAOS 355
#define T_HESIOD 356
#define T_TYPE 357
#define T_MASTER 358
#define T_SLAVE 359
#define T_STUB 360
#define T_RESPONSE 361
#define T_HINT 362
#define T_MASTERS 363
#define T_TRANSFER_SOURCE 364
#define T_PUBKEY 365
#define T_ALSO_NOTIFY 366
#define T_DIALUP 367
#define T_FILE_IXFR 368
#define T_IXFR_TMP 369
#define T_TRUSTED_KEYS 370
#define T_ACL 371
#define T_ALLOW_UPDATE 372
#define T_ALLOW_QUERY 373
#define T_ALLOW_TRANSFER 374
#define T_ALLOW_RECURSION 375
#define T_BLACKHOLE 376
#define T_SEC_KEY 377
#define T_ALGID 378
#define T_SECRET 379
#define T_CHECK_NAMES 380
#define T_WARN 381
#define T_FAIL 382
#define T_IGNORE 383
#define T_FORWARD 384
#define T_FORWARDERS 385
#define T_ONLY 386
#define T_FIRST 387
#define T_IF_NO_ANSWER 388
#define T_IF_NO_DOMAIN 389
#define T_YES 390
#define T_TRUE 391
#define T_NO 392
#define T_FALSE 393
const short yylhs[] = {                                        -1,
    0,   31,   31,   32,   32,   32,   32,   32,   32,   32,
   32,   32,   32,   32,   32,   33,   42,   34,   43,   43,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   46,
   44,   44,   44,   44,   44,   44,   44,   49,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   44,   44,   44,   44,   44,   44,   44,   44,   44,
   44,   35,   53,   53,   54,   54,   54,   54,   54,   54,
   15,   15,   12,   12,   13,   13,   14,   14,   16,    6,
    6,    5,    5,    4,    4,   56,   57,   48,   48,   48,
   48,    2,    2,    3,    3,   29,   29,   29,   29,   29,
   27,   27,   27,   28,   28,   28,   45,   45,   45,   45,
   51,   51,   51,   51,   26,   26,   26,   26,   52,   52,
   52,   47,   47,   58,   58,   59,   50,   50,   60,   60,
   61,   62,   36,   63,   63,   63,   65,   64,   67,   64,
   69,   69,   69,   69,   70,   70,   71,   72,   72,   72,
   72,   72,   73,   10,   10,   11,   11,   74,   75,   75,
   75,   75,   75,   75,   75,   68,   68,   68,    9,    9,
   76,   66,   66,   66,    8,    8,    8,    7,   77,   37,
   78,   78,   79,   79,   79,   79,   79,   79,   79,   20,
   20,   18,   18,   18,   17,   17,   17,   17,   17,   19,
   23,   81,   80,   80,   80,   82,   55,   55,   55,   83,
   41,   84,   84,   84,   24,   25,   40,   86,   38,   85,
   85,   21,   21,   22,   22,   22,   22,   22,   87,   87,
   88,   88,   88,   88,   88,   88,   88,   88,   88,   88,
   88,   91,   88,   88,   88,   88,   88,   88,   88,   88,
   88,   88,   88,   89,   89,   94,   94,   93,   93,   95,
   95,   96,   90,   90,   92,   92,   97,   97,   98,   39,
   99,   99,  100,  100,    1,   30,   30,
};
const short yylen[] = {                                         2,
    1,    1,    2,    1,    2,    2,    2,    2,    2,    2,
    2,    2,    1,    2,    2,    3,    0,    5,    2,    3,
    0,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    3,    2,    2,    5,    2,    0,
    5,    2,    2,    4,    4,    4,    4,    0,    5,    4,
    4,    1,    1,    2,    2,    2,    2,    2,    2,    2,
    2,    2,    2,    2,    2,    4,    2,    2,    0,    2,
    1,    4,    2,    3,    0,    8,   10,   12,    8,    1,
    2,    3,    0,    2,    0,    2,    0,    2,    5,    1,
    1,    1,    1,    1,    1,    2,    2,    1,    1,    2,
    2,    0,    2,    0,    2,    1,    1,    1,    1,    1,
    1,    1,    1,    1,    1,    1,    1,    1,    1,    1,
    2,    2,    2,    2,    1,    1,    1,    1,    2,    2,
    2,    0,    1,    2,    3,    1,    0,    1,    2,    3,
    1,    0,    5,    2,    3,    1,    0,    6,    0,    6,
    1,    1,    2,    1,    2,    2,    2,    0,    1,    1,
    2,    2,    3,    1,    1,    0,    1,    2,    1,    1,
    1,    2,    2,    2,    2,    2,    3,    1,    1,    1,
    1,    2,    3,    1,    1,    1,    1,    1,    0,    6,
    2,    3,    2,    2,    2,    2,    4,    2,    1,    2,
    3,    1,    2,    2,    1,    3,    3,    1,    3,    1,
    1,    1,    2,    3,    1,    1,    2,    3,    1,    0,
    6,    2,    2,    1,    3,    3,    5,    0,    5,    0,
    3,    0,    1,    1,    1,    1,    1,    1,    2,    3,
    2,    2,    2,    2,    5,    2,    2,    4,    4,    4,
    2,    0,    5,    2,    2,    2,    2,    2,    5,    5,
    4,    2,    1,    2,    3,    1,    3,    0,    1,    2,
    3,    1,    1,    1,    0,    1,    2,    3,    1,    4,
    2,    3,    5,    5,    1,    1,    1,
};
const short yydefred[] = {                                      0,
    0,   13,    0,   17,    0,  152,    0,    0,    0,    0,
  230,    0,    0,    2,    4,    0,    0,    0,    0,    0,
    0,    0,    0,   14,   15,    0,    0,    0,    0,  199,
    0,    0,  296,  297,    0,    0,    3,    5,    6,    7,
    8,    9,   10,   11,   12,   16,    0,   90,    0,    0,
    0,    0,    0,    0,  238,  243,    0,    0,    0,    0,
    0,   81,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,   58,    0,    0,    0,    0,    0,    0,
    0,   50,    0,    0,   62,   63,  102,  103,    0,    0,
   82,    0,   83,  156,    0,    0,    0,    0,    0,    0,
    0,    0,  290,    0,  291,    0,    0,    0,    0,    0,
  212,    0,  218,    0,  220,    0,   24,   26,   25,   29,
   27,   28,  120,  116,  117,  118,  119,   31,   32,   33,
   34,   30,    0,    0,   52,    0,    0,    0,    0,    0,
  136,  137,  138,  131,  135,  132,  133,  134,   23,   22,
   38,   70,   39,  139,  140,  141,  100,  101,   64,   65,
   66,   35,   36,   37,   43,   44,   40,   41,   67,   68,
   69,   71,   74,   46,   72,   42,   47,   73,   78,   77,
   80,    0,    0,   53,    0,   75,    0,    0,    0,    0,
  121,  122,  123,    0,  127,  128,  129,  130,   49,    0,
   18,    0,   19,    0,    0,    0,   84,  196,  197,  157,
  198,  195,  190,  159,  189,  153,    0,  154,  209,    0,
    0,    0,    0,    0,    0,    0,    0,    0,  239,    0,
    0,  292,    0,    0,  214,    0,  213,  210,  237,    0,
  234,    0,    0,    0,    0,    0,  295,  105,  104,  107,
  106,  110,  111,  113,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  124,  125,  126,   45,
    0,   20,    0,    0,    0,    0,    0,  155,  206,  203,
  205,    0,  204,  208,  200,    0,  201,  273,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  262,    0,    0,    0,
    0,  216,  217,  219,  211,    0,    0,  232,  233,  231,
    0,   94,    0,    0,   76,    0,   91,   57,   61,  151,
    0,    0,    0,   54,   56,   55,   60,  146,    0,    0,
    0,    0,    0,    0,    0,    0,  225,  222,  221,    0,
    0,  202,  264,  266,  267,  268,  265,  252,  244,  245,
  247,  246,  248,  251,    0,    0,  256,    0,    0,    0,
  272,  253,  254,    0,    0,    0,  257,  283,  284,  261,
    0,  241,    0,  249,  293,  294,  235,  236,   48,   96,
    0,    0,   92,   59,    0,  149,   51,    0,  144,    0,
    0,    0,  194,  191,    0,    0,  188,    0,    0,    0,
  181,    0,    0,    0,    0,  179,  180,    0,  207,    0,
  223,  115,    0,    0,    0,  282,    0,    0,    0,    0,
    0,    0,    0,  250,   98,    0,  150,  145,    0,    0,
    0,  158,    0,  192,  164,    0,  161,  182,    0,  175,
  177,  178,  174,  183,  184,  185,  160,    0,  186,  224,
    0,    0,    0,    0,    0,  271,    0,  280,  258,  259,
  260,  289,    0,    0,    0,   99,    0,    0,   89,  193,
  163,    0,    0,    0,    0,  173,  187,    0,  255,    0,
  274,  269,  270,  281,  263,    0,  287,    0,  229,  226,
    0,    0,  165,  166,  167,  171,  172,  277,  275,  288,
    0,   87,    0,  227,    0,  228,   88,
};
const short yydgoto[] = {                                      12,
  289,  179,  406,  290,  129,  199,  250,  251,  444,  491,
  492,  297,  364,  432,  298,  299,  151,  152,  153,  154,
   55,  404,  540,  284,  285,  184,  234,  310,  168,  155,
   13,   14,   15,   16,   17,   18,   19,   20,   21,   22,
   23,   27,  123,  124,  239,  240,  379,  175,  225,  371,
  125,  126,   51,   52,  541,  176,  177,  380,  381,  372,
  373,   29,  137,  138,  316,  445,  317,  455,  488,  524,
  525,  526,  456,  457,  458,  446,   54,  266,  267,  390,
  391,  542,   36,  286,  269,  140,  348,  349,  502,  420,
  421,  513,  467,  503,  468,  469,  514,  515,   58,   59,
};
const short yysindex[] = {                                    128,
 -167,    0, -213,    0,  -60,    0, -181, -179,  -26, -187,
    0,    0,  128,    0,    0, -156, -154, -152, -150, -117,
 -104,  -95,  -91,    0,    0,  -89,   23, -170,   32,    0,
 -187, -103,    0,    0,   36, -187,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  403,    0,   -7,  -84,
  -92,  -83, -239,   56,    0,    0,  -68,  -88,  -59,    1,
   63,    0,  -52,  -46,  -43,  -16,   10,   52,  -62,  -62,
  -62,  -62,   -6,   47,    6,   78,  142,  142,  142,  142,
   60,   62,  -62,   68,  -62,   80,   88,   96,  -51,  100,
  102,  -55,  -62,  -62,  -62,  -62,  -62,  106,  117,  118,
  142,  120,  -62,  123,  -62,  -62,  124,  126,  -62,  134,
  239,  271,   -7,    0,  -62,  273,  277,  288,  293, -148,
  -69,    0,  282,  148,    0,    0,    0,    0, -191,  130,
    0,  163,    0,    0,  103, -201, -111,  164,   93,  299,
  165,  166,    0,  169,    0,  381,  382,  170,    1, -110,
    0,  174,    0,  -29,    0, -241,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  -31,   -7,    0,  154,  155,  180,  324,  167,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    1,    1,    0,  325,    0,    1,    1,    1,    1,
    0,    0,    0, -263,    0,    0,    0,    0,    0,  326,
    0,  194,    0,  180,  329,  195,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  196,    0,    0,  -51,
  -62,  197,  334,  -62,  -62,   19,  202,  429,    0,  201,
  209,    0,  210,  214,    0,  -25,    0,    0,    0,  218,
    0, -187, -187,   97,   99,  354,    0,    0,    0,    0,
    0,    0,    0,    0,    1, -187,  127, -102,  223,  -21,
  -17,  227,  -11,    9,   18,   22,    0,    0,    0,    0,
  229,    0,  199,    1,  204,  367,  374,    0,    0,    0,
    0, -172,    0,    0,    0,  243,    0,    0,  242,  -47,
  -62,  142,  241, -249,  225,   -7,   74,  383,  -62,  246,
  248,  387,  388,  389, -263,  -45,    0,   98,  247,  253,
  254,    0,    0,    0,    0,  259,  260,    0,    0,    0,
   29,    0, -187,  237,    0,  263,    0,    0,    0,    0,
  396,  227,  265,    0,    0,    0,    0,    0,  398,  229,
  267,  402,   33,  268, -180,  153,    0,    0,    0,  -78,
  269,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  180,  405,    0,  270,  274,  272,
    0,    0,    0,    1,    1,    1,    0,    0,    0,    0,
  409,    0,  278,    0,    0,    0,    0,    0,    0,    0,
  275,  256,    0,    0,  283,    0,    0,  285,    0,    1,
  184,  250,    0,    0,   -8,  286,    0, -203,  300, -196,
    0,  -62,  -62,  -62, -116,    0,    0,  287,    0,  289,
    0,    0,  302,  304,  305,    0,  440,  272,  309,   38,
   42,   46,  310,    0,    0,  307,    0,    0,   50,  446,
  311,    0,  314,    0,    0,  313,    0,    0,   14,    0,
    0,    0,    0,    0,    0,    0,    0,  319,    0,    0,
  200,  -82,  333,  330,  332,    0,  355,    0,    0,    0,
    0,    0,  488,  310,  357,    0,  266,  -76,    0,    0,
    0, -234,  142,  280,  279,    0,    0, -187,    0,  361,
    0,    0,    0,    0,    0,  362,    0,  497,    0,    0,
  -73,  364,    0,    0,    0,    0,    0,    0,    0,    0,
  -76,    0,  365,    0,  -71,    0,    0,
};
const short yyrindex[] = {                                      0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  623,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  368,    0,    0,
  -87,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,  371,    0,    0,    0,
  368,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  501,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,  371,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  372,
    0,    0,    0,    0,    0,  373,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  375,  376,    0,    0, -236,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,   76, -236,    0,    0,
    0,  506,    0,    0,    0,    0,    0,    0,    0,    0,
  509,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,  512,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  356,    0,    0,    0,    0,    0,    0,
    0,  511,    0,    0,    0,    0,    0,    0,    0,  513,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  514,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,  384,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,  515,    0,    0,
    0,    0,  517,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,  386,    0,    0,  390,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
  393,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  519,    0,    0,  394,    0,    0,    0,
    0,    0,    0,  395,  397,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,    0,    0,    0,    0,
};
const short yygindex[] = {                                      0,
 -159,    0,    0,    0,  -41,  385,    0,    0,  524,    0,
    0,    0,    0,    0,    0,  363,  503,  -98,    0, -100,
    0,    0, -312,  378,  380,  -77,    0,  320,  -65,  -10,
    0,  668,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  559,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  633,  135,  510,  516,    0,  308,    0,
  317,    0,    0,  553,    0,    0,    0,    0,    0,  168,
  173,    0,    0,    0,  236,  291,    0,    0,  432,    0,
  321, -502,    0,    0,    0,    0,    0,  366,    0,    0,
    0,    0,    0,  231,    0,  244,    0,  220,    0,  679,
};
#define YYTABLESIZE 814
const short yytable[] = {                                      35,
  186,  187,  188,  150,  169,  170,  171,  150,  497,  388,
  288,  150,  149,  256,  281,  150,  134,  191,  294,  193,
   56,  150,  365,  212,  543,   61,  203,  204,  205,  206,
  207,  208,  131,  150,  128,  242,  143,  214,  553,  216,
  217,  150,  529,  220,   93,   93,  459,   26,  276,  226,
  150,  552,  553,  557,  150,  280,   33,   34,   33,   34,
  544,  150,   28,   33,   34,  150,  185,  185,  185,  185,
  150,  224,   33,   34,  150,  443,   30,  388,  150,   33,
   34,   31,  150,  387,  313,   48,  244,   33,   34,   24,
  185,  135,  136,  149,   25,  279,   32,  149,  245,  354,
   38,  149,   39,  368,   40,  149,   41,  369,  399,  400,
  401,  149,  402,  374,   49,   50,  482,  307,  308,  309,
   93,  300,  301,  149,  252,  255,  303,  304,  305,  306,
  485,  149,  291,  375,  403,  486,  282,  283,  253,   42,
  149,  490,  376,  325,  149,   47,  377,  146,  147,   33,
   34,  149,   43,  429,   53,  149,   57,  441,   60,  253,
  149,   44,  509,   48,  149,   45,  510,   46,  149,  242,
  511,   57,  149,  133,  517,  501,  130,  280,  139,  539,
  296,   33,   34,   33,   34,  156,   33,   34,   33,   34,
  141,  142,   49,   50,  361,  320,  163,  145,  323,  324,
  180,  280,  280,  163,  280,  280,  280,  280,  157,  231,
  232,  163,  233,  383,  158,  548,  448,  159,  449,  135,
  136,  450,  422,  451,  452,  453,  454,  287,  146,  147,
   33,   34,  146,  147,   33,   34,  146,  147,   33,   34,
  146,  147,   33,   34,  160,  462,  146,  147,   33,   34,
  127,   33,   34,  172,  397,  202,  197,  198,  146,  147,
   33,   34,  280,  394,  395,  396,  146,  147,   33,   34,
  161,  356,  357,  411,  259,  146,  147,   33,   34,  146,
  147,   33,   34,  178,  280,  362,  146,  147,   33,   34,
  146,  147,   33,   34,  407,  146,  147,   33,   34,  146,
  147,   33,   34,  146,  147,   33,   34,  146,  147,   33,
   34,  389,  162,  470,  471,  472,  235,  236,  237,  238,
  189,  185,  190,  260,  173,  174,  192,  164,  165,  166,
  167,  253,  408,  409,  164,  165,  166,  167,  194,  479,
  418,  419,  164,  165,  166,  167,  195,  148,  259,  522,
  523,  148,  430,  328,  196,  148,   95,   95,  200,  148,
  201,  222,   33,   34,  209,  148,  261,  262,  263,  264,
  265,  280,  280,  280,  255,  210,  211,  148,  213,  389,
  280,  215,  218,    1,  219,  148,  494,  495,  496,    2,
    3,    4,  221,  223,  148,  227,  248,  260,  148,  228,
  181,   33,   34,  329,  243,  148,  241,  330,  447,  148,
  229,    5,  249,  331,  148,  230,  246,  332,  148,  247,
  258,  268,  148,  270,  271,  272,  148,  273,  274,  275,
  278,  173,  333,  174,  255,  182,  183,  487,  287,  493,
  261,  262,  263,  264,  265,  545,  295,  302,  311,  296,
  312,  314,  318,  315,  334,  321,  322,    6,  327,  350,
  335,  336,  337,  338,  339,  340,  341,  351,  352,  342,
  343,  344,  353,    7,  355,  283,  282,  345,  360,  367,
    8,  346,  347,  363,  370,  448,  378,  449,  382,  385,
  450,  384,  451,  452,  453,  454,  386,    9,   10,  392,
  393,  398,  405,  424,   11,  410,  412,  389,  413,  414,
  415,  416,  185,  425,  426,  427,  428,  389,  431,  433,
  434,  436,  437,  439,  440,  461,  442,  463,  464,  466,
  389,  473,  465,  480,  474,  475,  476,   62,  481,  477,
  389,  478,  484,  499,  389,  500,   63,   64,   65,   66,
   67,   68,   69,   70,   71,   72,   73,   74,   75,  501,
  489,   76,  504,  505,  506,  508,  516,  512,  518,  519,
  520,  521,   77,   78,   79,  527,  528,   80,   81,   82,
   83,   84,   85,   86,   87,   88,   89,   90,   91,  531,
  532,   92,  533,   93,   94,   95,   96,   97,   98,   99,
  100,  101,  102,  103,  104,  105,  106,  107,  108,  109,
  110,  534,  535,  537,  522,  538,  523,  549,  550,  551,
  554,  556,    1,  112,   85,  111,  112,   21,  240,  215,
  147,  108,  109,  142,  114,  148,   97,  143,  278,  279,
  176,  285,  162,  286,  319,  113,  168,  114,  115,  276,
   86,  169,  277,  170,  116,  117,  118,  119,   62,  254,
  366,  120,  359,  358,  417,  121,  122,   63,   64,   65,
   66,   67,   68,   69,   70,   71,   72,   73,   74,   75,
   37,  242,   76,  132,  328,  555,  293,  438,  435,  257,
  498,  292,  547,   77,   78,   79,  546,  326,   80,   81,
   82,   83,   84,   85,   86,   87,   88,   89,   90,   91,
  460,  507,   92,  423,   93,   94,   95,   96,   97,   98,
   99,  100,  101,  102,  103,  104,  105,  106,  107,  108,
  109,  110,  530,  536,  329,  483,  144,    0,  330,    0,
    0,    0,    0,    0,  331,    0,  111,  112,  332,    0,
    0,    0,    0,    0,    0,    0,    0,    0,    0,    0,
    0,    0,    0,  333,    0,    0,  113,    0,  114,  115,
    0,    0,    0,    0,    0,  116,  117,  118,  119,    0,
    0,    0,  120,    0,    0,  334,  121,  122,    0,    0,
    0,  335,  336,  337,  338,  339,  340,  341,    0,    0,
  342,  343,  344,    0,    0,    0,    0,    0,  345,    0,
    0,    0,  346,  347,
};
const short yycheck[] = {                                      10,
   78,   79,   80,   33,   70,   71,   72,   33,  125,  322,
   42,   33,  123,  125,  256,   33,  256,   83,  178,   85,
   31,   33,  125,  101,  259,   36,   92,   93,   94,   95,
   96,   97,  125,   33,   42,  123,  125,  103,  541,  105,
  106,   33,  125,  109,  281,  282,  125,  261,  149,  115,
   33,  125,  555,  125,   33,  154,  260,  261,  260,  261,
  295,   33,  123,  260,  261,   33,   77,   78,   79,   80,
   33,  113,  260,  261,   33,  256,  258,  390,   33,  260,
  261,  261,   33,  256,  244,  256,  278,  260,  261,  257,
  101,  331,  332,  123,  262,  125,  123,  123,  290,  125,
  257,  123,  257,  125,  257,  123,  257,  125,  358,  359,
  360,  123,  362,  125,  285,  286,  125,  381,  382,  383,
  357,  222,  223,  123,  135,  136,  227,  228,  229,  230,
  334,  123,  174,  125,  384,  339,  378,  379,  340,  257,
  123,  338,  125,  125,  123,  123,  125,  258,  259,  260,
  261,  123,  257,  125,  123,  123,  260,  125,  123,  340,
  123,  257,  125,  256,  123,  257,  125,  257,  123,  257,
  125,  260,  123,  257,  125,  258,  261,  276,  123,  256,
  283,  260,  261,  260,  261,  123,  260,  261,  260,  261,
  259,  260,  285,  286,  295,  261,  259,  257,  264,  265,
  123,  300,  301,  259,  303,  304,  305,  306,  261,  358,
  359,  259,  361,  314,  261,  528,  333,  261,  335,  331,
  332,  338,  125,  340,  341,  342,  343,  259,  258,  259,
  260,  261,  258,  259,  260,  261,  258,  259,  260,  261,
  258,  259,  260,  261,  261,  405,  258,  259,  260,  261,
  258,  260,  261,  260,  332,  311,  308,  309,  258,  259,
  260,  261,  361,  311,  330,  331,  258,  259,  260,  261,
  261,  282,  283,  339,  256,  258,  259,  260,  261,  258,
  259,  260,  261,  278,  383,  296,  258,  259,  260,  261,
  258,  259,  260,  261,  336,  258,  259,  260,  261,  258,
  259,  260,  261,  258,  259,  260,  261,  258,  259,  260,
  261,  322,  261,  414,  415,  416,  386,  387,  388,  389,
  261,  332,  261,  305,  278,  279,  259,  390,  391,  392,
  393,  340,  259,  260,  390,  391,  392,  393,  259,  440,
  386,  387,  390,  391,  392,  393,  259,  377,  256,  336,
  337,  377,  363,  256,  259,  377,  281,  282,  259,  377,
  259,  123,  260,  261,  259,  377,  348,  349,  350,  351,
  352,  470,  471,  472,  385,  259,  259,  377,  259,  390,
  479,  259,  259,  256,  259,  377,  452,  453,  454,  262,
  263,  264,  259,  123,  377,  123,  294,  305,  377,  123,
  259,  260,  261,  306,  257,  377,  125,  310,  256,  377,
  123,  284,  310,  316,  377,  123,  287,  320,  377,  257,
  257,  123,  377,  259,  259,  257,  377,   47,   47,  260,
  257,  278,  335,  279,  445,  294,  295,  448,  259,  450,
  348,  349,  350,  351,  352,  523,  123,  123,  123,  283,
  257,  123,  257,  259,  357,  259,  123,  330,  257,  259,
  363,  364,  365,  366,  367,  368,  369,  259,  259,  372,
  373,  374,  259,  346,  257,  379,  378,  380,  125,  257,
  353,  384,  385,  357,  258,  333,  258,  335,  290,  123,
  338,  288,  340,  341,  342,  343,  123,  370,  371,  257,
  259,  261,  278,  257,  377,  123,  261,  518,  261,  123,
  123,  123,  523,  261,  261,  257,  257,  528,  282,  257,
  125,  257,  125,  257,  123,  257,  259,  123,  259,  258,
  541,  123,  259,  350,  257,  261,  281,  256,  289,  257,
  551,  257,  257,  257,  555,  257,  265,  266,  267,  268,
  269,  270,  271,  272,  273,  274,  275,  276,  277,  258,
  261,  280,  259,  259,  125,  257,  260,  258,  123,  259,
  257,  259,  291,  292,  293,  257,  377,  296,  297,  298,
  299,  300,  301,  302,  303,  304,  305,  306,  307,  257,
  261,  310,  261,  312,  313,  314,  315,  316,  317,  318,
  319,  320,  321,  322,  323,  324,  325,  326,  327,  328,
  329,  257,  125,  257,  336,  350,  337,  257,  257,  123,
  257,  257,    0,  123,  257,  344,  345,  257,  257,  257,
  125,  257,  257,  125,  123,  125,  281,  125,  125,  125,
  257,  125,  257,  125,  260,  364,  257,  366,  367,  257,
  257,  257,  150,  257,  373,  374,  375,  376,  256,  136,
  298,  380,  285,  284,  345,  384,  385,  265,  266,  267,
  268,  269,  270,  271,  272,  273,  274,  275,  276,  277,
   13,  123,  280,   51,  256,  551,  177,  380,  372,  137,
  455,  176,  525,  291,  292,  293,  524,  266,  296,  297,
  298,  299,  300,  301,  302,  303,  304,  305,  306,  307,
  390,  468,  310,  348,  312,  313,  314,  315,  316,  317,
  318,  319,  320,  321,  322,  323,  324,  325,  326,  327,
  328,  329,  502,  514,  306,  445,   58,   -1,  310,   -1,
   -1,   -1,   -1,   -1,  316,   -1,  344,  345,  320,   -1,
   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,   -1,
   -1,   -1,   -1,  335,   -1,   -1,  364,   -1,  366,  367,
   -1,   -1,   -1,   -1,   -1,  373,  374,  375,  376,   -1,
   -1,   -1,  380,   -1,   -1,  357,  384,  385,   -1,   -1,
   -1,  363,  364,  365,  366,  367,  368,  369,   -1,   -1,
  372,  373,  374,   -1,   -1,   -1,   -1,   -1,  380,   -1,
   -1,   -1,  384,  385,
};
#define YYFINAL 12
#ifndef YYDEBUG
#define YYDEBUG 0
#endif
#define YYMAXTOKEN 393
#if YYDEBUG
const char * const yyname[] = {
"end-of-file",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
"'!'",0,0,0,0,0,0,0,0,"'*'",0,0,0,0,"'/'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"'{'",0,"'}'",0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,"L_EOS",
"L_IPADDR","L_NUMBER","L_STRING","L_QSTRING","L_END_INCLUDE","T_INCLUDE",
"T_OPTIONS","T_DIRECTORY","T_PIDFILE","T_NAMED_XFER","T_DUMP_FILE",
"T_STATS_FILE","T_MEMSTATS_FILE","T_FAKE_IQUERY","T_RECURSION","T_FETCH_GLUE",
"T_HITCOUNT","T_PREFERRED_GLUE","T_QUERY_SOURCE","T_LISTEN_ON","T_PORT",
"T_ADDRESS","T_RRSET_ORDER","T_ORDER","T_NAME","T_CLASS","T_CONTROLS","T_INET",
"T_UNIX","T_PERM","T_OWNER","T_GROUP","T_ALLOW","T_DATASIZE","T_STACKSIZE",
"T_CORESIZE","T_DEFAULT","T_UNLIMITED","T_FILES","T_VERSION","T_HOSTNAME",
"T_HOSTSTATS","T_HOSTSTATSMAX","T_DEALLOC_ON_EXIT","T_TRANSFERS_IN",
"T_TRANSFERS_OUT","T_TRANSFERS_PER_NS","T_TRANSFER_FORMAT",
"T_MAX_TRANSFER_TIME_IN","T_SERIAL_QUERIES","T_ONE_ANSWER","T_MANY_ANSWERS",
"T_NOTIFY","T_EXPLICIT","T_NOTIFY_INITIAL","T_AUTH_NXDOMAIN",
"T_MULTIPLE_CNAMES","T_USE_IXFR","T_MAINTAIN_IXFR_BASE","T_CLEAN_INTERVAL",
"T_INTERFACE_INTERVAL","T_STATS_INTERVAL","T_MAX_LOG_SIZE_IXFR","T_HEARTBEAT",
"T_USE_ID_POOL","T_MAX_NCACHE_TTL","T_HAS_OLD_CLIENTS","T_RFC2308_TYPE1",
"T_LAME_TTL","T_MIN_ROOTS","T_TREAT_CR_AS_SPACE","T_EDNS_UDP_SIZE","T_LOGGING",
"T_CATEGORY","T_CHANNEL","T_SEVERITY","T_DYNAMIC","T_FILE","T_VERSIONS",
"T_SIZE","T_SYSLOG","T_DEBUG","T_NULL_OUTPUT","T_PRINT_TIME","T_PRINT_CATEGORY",
"T_PRINT_SEVERITY","T_SORTLIST","T_TOPOLOGY","T_SERVER","T_LONG_AXFR","T_BOGUS",
"T_TRANSFERS","T_KEYS","T_SUPPORT_IXFR","T_EDNS","T_ZONE","T_IN","T_CHAOS",
"T_HESIOD","T_TYPE","T_MASTER","T_SLAVE","T_STUB","T_RESPONSE","T_HINT",
"T_MASTERS","T_TRANSFER_SOURCE","T_PUBKEY","T_ALSO_NOTIFY","T_DIALUP",
"T_FILE_IXFR","T_IXFR_TMP","T_TRUSTED_KEYS","T_ACL","T_ALLOW_UPDATE",
"T_ALLOW_QUERY","T_ALLOW_TRANSFER","T_ALLOW_RECURSION","T_BLACKHOLE",
"T_SEC_KEY","T_ALGID","T_SECRET","T_CHECK_NAMES","T_WARN","T_FAIL","T_IGNORE",
"T_FORWARD","T_FORWARDERS","T_ONLY","T_FIRST","T_IF_NO_ANSWER","T_IF_NO_DOMAIN",
"T_YES","T_TRUE","T_NO","T_FALSE",
};
const char * const yyrule[] = {
"$accept : config_file",
"config_file : statement_list",
"statement_list : statement",
"statement_list : statement_list statement",
"statement : include_stmt",
"statement : options_stmt L_EOS",
"statement : controls_stmt L_EOS",
"statement : logging_stmt L_EOS",
"statement : server_stmt L_EOS",
"statement : zone_stmt L_EOS",
"statement : trusted_keys_stmt L_EOS",
"statement : acl_stmt L_EOS",
"statement : key_stmt L_EOS",
"statement : L_END_INCLUDE",
"statement : error L_EOS",
"statement : error L_END_INCLUDE",
"include_stmt : T_INCLUDE L_QSTRING L_EOS",
"$$1 :",
"options_stmt : T_OPTIONS $$1 '{' options '}'",
"options : option L_EOS",
"options : options option L_EOS",
"option :",
"option : T_HOSTNAME L_QSTRING",
"option : T_VERSION L_QSTRING",
"option : T_DIRECTORY L_QSTRING",
"option : T_NAMED_XFER L_QSTRING",
"option : T_PIDFILE L_QSTRING",
"option : T_STATS_FILE L_QSTRING",
"option : T_MEMSTATS_FILE L_QSTRING",
"option : T_DUMP_FILE L_QSTRING",
"option : T_PREFERRED_GLUE L_STRING",
"option : T_FAKE_IQUERY yea_or_nay",
"option : T_RECURSION yea_or_nay",
"option : T_FETCH_GLUE yea_or_nay",
"option : T_HITCOUNT yea_or_nay",
"option : T_NOTIFY T_EXPLICIT",
"option : T_NOTIFY yea_or_nay",
"option : T_NOTIFY_INITIAL yea_or_nay",
"option : T_HOSTSTATS yea_or_nay",
"option : T_DEALLOC_ON_EXIT yea_or_nay",
"option : T_USE_IXFR yea_or_nay",
"option : T_MAINTAIN_IXFR_BASE yea_or_nay",
"option : T_HAS_OLD_CLIENTS yea_or_nay",
"option : T_AUTH_NXDOMAIN yea_or_nay",
"option : T_MULTIPLE_CNAMES yea_or_nay",
"option : T_CHECK_NAMES check_names_type check_names_opt",
"option : T_USE_ID_POOL yea_or_nay",
"option : T_RFC2308_TYPE1 yea_or_nay",
"option : T_LISTEN_ON maybe_port '{' address_match_list '}'",
"option : T_FORWARD forward_opt",
"$$2 :",
"option : T_FORWARDERS $$2 '{' opt_forwarders_list '}'",
"option : T_QUERY_SOURCE query_source",
"option : T_TRANSFER_SOURCE maybe_wild_addr",
"option : T_ALLOW_QUERY '{' address_match_list '}'",
"option : T_ALLOW_RECURSION '{' address_match_list '}'",
"option : T_ALLOW_TRANSFER '{' address_match_list '}'",
"option : T_SORTLIST '{' address_match_list '}'",
"$$3 :",
"option : T_ALSO_NOTIFY $$3 '{' opt_also_notify_list '}'",
"option : T_BLACKHOLE '{' address_match_list '}'",
"option : T_TOPOLOGY '{' address_match_list '}'",
"option : size_clause",
"option : transfer_clause",
"option : T_TRANSFER_FORMAT transfer_format",
"option : T_MAX_TRANSFER_TIME_IN L_NUMBER",
"option : T_SERIAL_QUERIES L_NUMBER",
"option : T_CLEAN_INTERVAL L_NUMBER",
"option : T_INTERFACE_INTERVAL L_NUMBER",
"option : T_STATS_INTERVAL L_NUMBER",
"option : T_HOSTSTATSMAX L_NUMBER",
"option : T_MAX_LOG_SIZE_IXFR size_spec",
"option : T_MAX_NCACHE_TTL L_NUMBER",
"option : T_LAME_TTL L_NUMBER",
"option : T_HEARTBEAT L_NUMBER",
"option : T_DIALUP yea_or_nay",
"option : T_RRSET_ORDER '{' rrset_ordering_list '}'",
"option : T_TREAT_CR_AS_SPACE yea_or_nay",
"option : T_MIN_ROOTS L_NUMBER",
"option :",
"option : T_EDNS_UDP_SIZE L_NUMBER",
"option : error",
"controls_stmt : T_CONTROLS '{' controls '}'",
"controls : control L_EOS",
"controls : controls control L_EOS",
"control :",
"control : T_INET maybe_wild_addr T_PORT in_port T_ALLOW '{' address_match_list '}'",
"control : T_INET maybe_wild_addr T_ALLOW '{' address_match_list '}' T_KEYS '{' dummy_key_list '}'",
"control : T_INET maybe_wild_addr T_PORT in_port T_ALLOW '{' address_match_list '}' T_KEYS '{' dummy_key_list '}'",
"control : T_UNIX L_QSTRING T_PERM L_NUMBER T_OWNER L_NUMBER T_GROUP L_NUMBER",
"control : error",
"rrset_ordering_list : rrset_ordering_element L_EOS",
"rrset_ordering_list : rrset_ordering_list rrset_ordering_element L_EOS",
"ordering_class :",
"ordering_class : T_CLASS any_string",
"ordering_type :",
"ordering_type : T_TYPE any_string",
"ordering_name :",
"ordering_name : T_NAME L_QSTRING",
"rrset_ordering_element : ordering_class ordering_type ordering_name T_ORDER L_STRING",
"transfer_format : T_ONE_ANSWER",
"transfer_format : T_MANY_ANSWERS",
"maybe_wild_addr : L_IPADDR",
"maybe_wild_addr : '*'",
"maybe_wild_port : in_port",
"maybe_wild_port : '*'",
"query_source_address : T_ADDRESS maybe_wild_addr",
"query_source_port : T_PORT maybe_wild_port",
"query_source : query_source_address",
"query_source : query_source_port",
"query_source : query_source_address query_source_port",
"query_source : query_source_port query_source_address",
"maybe_port :",
"maybe_port : T_PORT in_port",
"maybe_zero_port :",
"maybe_zero_port : T_PORT in_port",
"yea_or_nay : T_YES",
"yea_or_nay : T_TRUE",
"yea_or_nay : T_NO",
"yea_or_nay : T_FALSE",
"yea_or_nay : L_NUMBER",
"check_names_type : T_MASTER",
"check_names_type : T_SLAVE",
"check_names_type : T_RESPONSE",
"check_names_opt : T_WARN",
"check_names_opt : T_FAIL",
"check_names_opt : T_IGNORE",
"forward_opt : T_ONLY",
"forward_opt : T_FIRST",
"forward_opt : T_IF_NO_ANSWER",
"forward_opt : T_IF_NO_DOMAIN",
"size_clause : T_DATASIZE size_spec",
"size_clause : T_STACKSIZE size_spec",
"size_clause : T_CORESIZE size_spec",
"size_clause : T_FILES size_spec",
"size_spec : any_string",
"size_spec : L_NUMBER",
"size_spec : T_DEFAULT",
"size_spec : T_UNLIMITED",
"transfer_clause : T_TRANSFERS_IN L_NUMBER",
"transfer_clause : T_TRANSFERS_OUT L_NUMBER",
"transfer_clause : T_TRANSFERS_PER_NS L_NUMBER",
"opt_forwarders_list :",
"opt_forwarders_list : forwarders_in_addr_list",
"forwarders_in_addr_list : forwarders_in_addr L_EOS",
"forwarders_in_addr_list : forwarders_in_addr_list forwarders_in_addr L_EOS",
"forwarders_in_addr : L_IPADDR",
"opt_also_notify_list :",
"opt_also_notify_list : also_notify_in_addr_list",
"also_notify_in_addr_list : also_notify_in_addr L_EOS",
"also_notify_in_addr_list : also_notify_in_addr_list also_notify_in_addr L_EOS",
"also_notify_in_addr : L_IPADDR",
"$$4 :",
"logging_stmt : T_LOGGING $$4 '{' logging_opts_list '}'",
"logging_opts_list : logging_opt L_EOS",
"logging_opts_list : logging_opts_list logging_opt L_EOS",
"logging_opts_list : error",
"$$5 :",
"logging_opt : T_CATEGORY category $$5 '{' channel_list '}'",
"$$6 :",
"logging_opt : T_CHANNEL channel_name $$6 '{' channel_opt_list '}'",
"channel_severity : any_string",
"channel_severity : T_DEBUG",
"channel_severity : T_DEBUG L_NUMBER",
"channel_severity : T_DYNAMIC",
"version_modifier : T_VERSIONS L_NUMBER",
"version_modifier : T_VERSIONS T_UNLIMITED",
"size_modifier : T_SIZE size_spec",
"maybe_file_modifiers :",
"maybe_file_modifiers : version_modifier",
"maybe_file_modifiers : size_modifier",
"maybe_file_modifiers : version_modifier size_modifier",
"maybe_file_modifiers : size_modifier version_modifier",
"channel_file : T_FILE L_QSTRING maybe_file_modifiers",
"facility_name : any_string",
"facility_name : T_SYSLOG",
"maybe_syslog_facility :",
"maybe_syslog_facility : facility_name",
"channel_syslog : T_SYSLOG maybe_syslog_facility",
"channel_opt : channel_file",
"channel_opt : channel_syslog",
"channel_opt : T_NULL_OUTPUT",
"channel_opt : T_SEVERITY channel_severity",
"channel_opt : T_PRINT_TIME yea_or_nay",
"channel_opt : T_PRINT_CATEGORY yea_or_nay",
"channel_opt : T_PRINT_SEVERITY yea_or_nay",
"channel_opt_list : channel_opt L_EOS",
"channel_opt_list : channel_opt_list channel_opt L_EOS",
"channel_opt_list : error",
"channel_name : any_string",
"channel_name : T_NULL_OUTPUT",
"channel : channel_name",
"channel_list : channel L_EOS",
"channel_list : channel_list channel L_EOS",
"channel_list : error",
"category_name : any_string",
"category_name : T_DEFAULT",
"category_name : T_NOTIFY",
"category : category_name",
"$$7 :",
"server_stmt : T_SERVER L_IPADDR $$7 '{' server_info_list '}'",
"server_info_list : server_info L_EOS",
"server_info_list : server_info_list server_info L_EOS",
"server_info : T_BOGUS yea_or_nay",
"server_info : T_SUPPORT_IXFR yea_or_nay",
"server_info : T_TRANSFERS L_NUMBER",
"server_info : T_TRANSFER_FORMAT transfer_format",
"server_info : T_KEYS '{' key_list '}'",
"server_info : T_EDNS yea_or_nay",
"server_info : error",
"address_match_list : address_match_element L_EOS",
"address_match_list : address_match_list address_match_element L_EOS",
"address_match_element : address_match_simple",
"address_match_element : '!' address_match_simple",
"address_match_element : T_SEC_KEY L_STRING",
"address_match_simple : L_IPADDR",
"address_match_simple : L_IPADDR '/' L_NUMBER",
"address_match_simple : L_NUMBER '/' L_NUMBER",
"address_match_simple : address_name",
"address_match_simple : '{' address_match_list '}'",
"address_name : any_string",
"key_ref : any_string",
"key_list_element : key_ref",
"key_list : key_list_element L_EOS",
"key_list : key_list key_list_element L_EOS",
"key_list : error",
"dummy_key_list_element : key_ref",
"dummy_key_list : dummy_key_list_element L_EOS",
"dummy_key_list : dummy_key_list dummy_key_list_element L_EOS",
"dummy_key_list : error",
"$$8 :",
"key_stmt : T_SEC_KEY $$8 any_string '{' key_definition '}'",
"key_definition : algorithm_id secret",
"key_definition : secret algorithm_id",
"key_definition : error",
"algorithm_id : T_ALGID any_string L_EOS",
"secret : T_SECRET any_string L_EOS",
"acl_stmt : T_ACL any_string '{' address_match_list '}'",
"$$9 :",
"zone_stmt : T_ZONE L_QSTRING optional_class $$9 optional_zone_options_list",
"optional_zone_options_list :",
"optional_zone_options_list : '{' zone_option_list '}'",
"optional_class :",
"optional_class : any_string",
"zone_type : T_MASTER",
"zone_type : T_SLAVE",
"zone_type : T_HINT",
"zone_type : T_STUB",
"zone_type : T_FORWARD",
"zone_option_list : zone_option L_EOS",
"zone_option_list : zone_option_list zone_option L_EOS",
"zone_option : T_TYPE zone_type",
"zone_option : T_FILE L_QSTRING",
"zone_option : T_FILE_IXFR L_QSTRING",
"zone_option : T_IXFR_TMP L_QSTRING",
"zone_option : T_MASTERS maybe_zero_port '{' master_in_addr_list '}'",
"zone_option : T_TRANSFER_SOURCE maybe_wild_addr",
"zone_option : T_CHECK_NAMES check_names_opt",
"zone_option : T_ALLOW_UPDATE '{' address_match_list '}'",
"zone_option : T_ALLOW_QUERY '{' address_match_list '}'",
"zone_option : T_ALLOW_TRANSFER '{' address_match_list '}'",
"zone_option : T_FORWARD zone_forward_opt",
"$$10 :",
"zone_option : T_FORWARDERS $$10 '{' opt_zone_forwarders_list '}'",
"zone_option : T_MAX_TRANSFER_TIME_IN L_NUMBER",
"zone_option : T_MAX_LOG_SIZE_IXFR size_spec",
"zone_option : T_NOTIFY T_EXPLICIT",
"zone_option : T_NOTIFY yea_or_nay",
"zone_option : T_MAINTAIN_IXFR_BASE yea_or_nay",
"zone_option : T_PUBKEY L_NUMBER L_NUMBER L_NUMBER L_QSTRING",
"zone_option : T_PUBKEY L_STRING L_NUMBER L_NUMBER L_QSTRING",
"zone_option : T_ALSO_NOTIFY '{' opt_notify_in_addr_list '}'",
"zone_option : T_DIALUP yea_or_nay",
"zone_option : error",
"master_in_addr_list : master_in_addr L_EOS",
"master_in_addr_list : master_in_addr_list master_in_addr L_EOS",
"master_in_addr : L_IPADDR",
"master_in_addr : L_IPADDR T_SEC_KEY key_ref",
"opt_notify_in_addr_list :",
"opt_notify_in_addr_list : notify_in_addr_list",
"notify_in_addr_list : notify_in_addr L_EOS",
"notify_in_addr_list : notify_in_addr_list notify_in_addr L_EOS",
"notify_in_addr : L_IPADDR",
"zone_forward_opt : T_ONLY",
"zone_forward_opt : T_FIRST",
"opt_zone_forwarders_list :",
"opt_zone_forwarders_list : zone_forwarders_in_addr_list",
"zone_forwarders_in_addr_list : zone_forwarders_in_addr L_EOS",
"zone_forwarders_in_addr_list : zone_forwarders_in_addr_list zone_forwarders_in_addr L_EOS",
"zone_forwarders_in_addr : L_IPADDR",
"trusted_keys_stmt : T_TRUSTED_KEYS '{' trusted_keys_list '}'",
"trusted_keys_list : trusted_key L_EOS",
"trusted_keys_list : trusted_keys_list trusted_key L_EOS",
"trusted_key : L_STRING L_NUMBER L_NUMBER L_NUMBER L_QSTRING",
"trusted_key : L_STRING L_STRING L_NUMBER L_NUMBER L_QSTRING",
"in_port : L_NUMBER",
"any_string : L_STRING",
"any_string : L_QSTRING",
};
#endif
#if YYDEBUG
#include <stdio.h>
#endif
#ifdef YYSTACKSIZE
#undef YYMAXDEPTH
#define YYMAXDEPTH YYSTACKSIZE
#else
#ifdef YYMAXDEPTH
#define YYSTACKSIZE YYMAXDEPTH
#else
#define YYSTACKSIZE 10000
#define YYMAXDEPTH 10000
#endif
#endif
#define YYINITSTACKSIZE 200
int yydebug;
int yynerrs;
int yyerrflag;
int yychar;
short *yyssp;
YYSTYPE *yyvsp;
YYSTYPE yyval;
YYSTYPE yylval;
short *yyss;
short *yysslim;
YYSTYPE *yyvs;
int yystacksize;
#line 1874 "ns_parser.y"

static char *
canonical_name(char *name) {
	char canonical[MAXDNAME];
	
	if (strlen(name) >= MAXDNAME)
		return (NULL);
	strcpy(canonical, name);
	if (makename(canonical, ".", sizeof canonical) < 0)
		return (NULL);
	return (savestr(canonical, 0));
}

static void
init_acls() {
	ip_match_element ime;
	ip_match_list iml;
	struct in_addr address;

	/* Create the predefined ACLs */

	address.s_addr = 0U;

	/* ACL "any" */
	ime = new_ip_match_pattern(address, 0);
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("any", iml);

	/* ACL "none" */
	ime = new_ip_match_pattern(address, 0);
	ip_match_negate(ime);
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("none", iml);

	/* ACL "localhost" */
	ime = new_ip_match_localhost();
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("localhost", iml);

	/* ACL "localnets" */
	ime = new_ip_match_localnets();
	iml = new_ip_match_list();
	add_to_ip_match_list(iml, ime);
	define_acl("localnets", iml);
}

static void
free_sym_value(int type, void *value) {
	ns_debug(ns_log_parser, 99, "free_sym_value: type %06x value %p",
		 type, value);
	type &= ~0xffff;
	switch (type) {
	case SYM_ACL:
		free_ip_match_list(value);
		break;
	case SYM_KEY:
		free_key_info(value);
		break;
	case SYM_CHANNEL:
		INSIST(log_free_channel(value) == 0);
		break;
	default:
		ns_panic(ns_log_parser, 1,
			 "unhandled case in free_sym_value()");
		/* NOTREACHED */
		break;
	}
}

static log_channel
lookup_channel(char *name) {
	symbol_value value;

	if (lookup_symbol(channeltab, name, SYM_CHANNEL, &value))
		return ((log_channel)(value.pointer));
	return (NULL);
}

static void
define_channel(const char *name, log_channel channel) {
	symbol_value value;

	value.pointer = channel;  
	INSIST(log_inc_references(channel) == 0);
	define_symbol(channeltab, name, SYM_CHANNEL, value, SYMBOL_FREE_VALUE);
}

static void
define_builtin_channels() {
	define_channel("default_syslog", syslog_channel);
	define_channel("default_debug", debug_channel);
	define_channel("default_stderr", stderr_channel);
	define_channel("null", null_channel);
}

static void
parser_setup() {
	seen_options = 0;
	logged_options_error = 0;
	seen_topology = 0;
	symtab = new_symbol_table(SYMBOL_TABLE_SIZE, NULL);
	if (authtab != NULL)
		free_symbol_table(authtab);
	authtab = new_symbol_table(AUTH_TABLE_SIZE, free_sym_value);
	if (channeltab != NULL)
		free_symbol_table(channeltab);
	channeltab = new_symbol_table(AUTH_TABLE_SIZE, free_sym_value);
	init_acls();
	define_builtin_channels();
	INIT_LIST(current_controls);
}

static void
parser_cleanup() {
	if (symtab != NULL)
		free_symbol_table(symtab);
	symtab = NULL;
	/*
	 * We don't clean up authtab here because the ip_match_lists are in
	 * use.
	 */
}

/*
 * Public Interface
 */

ip_match_list
lookup_acl(const char *name) {
	symbol_value value;

	if (lookup_symbol(authtab, name, SYM_ACL, &value))
		return ((ip_match_list)(value.pointer));
	return (NULL);
}

void
define_acl(const char *name, ip_match_list iml) {
	symbol_value value;

	INSIST(name != NULL);
	INSIST(iml != NULL);

	value.pointer = iml;
	define_symbol(authtab, name, SYM_ACL, value, SYMBOL_FREE_VALUE);
	ns_debug(ns_log_parser, 7, "acl %s", name);
	dprint_ip_match_list(ns_log_parser, iml, 2, "allow ", "deny ");
}

struct dst_key *
lookup_key(char *name) {
	symbol_value value;

	if (lookup_symbol(authtab, name, SYM_KEY, &value))
		return ((struct dst_key *)(value.pointer));
	return (NULL);
}

void
define_key(const char *name, struct dst_key *dst_key) {
	symbol_value value;

	INSIST(name != NULL);
	INSIST(dst_key != NULL);

	value.pointer = dst_key;
	define_symbol(authtab, name, SYM_KEY, value, SYMBOL_FREE_VALUE);
	dprint_key_info(dst_key);
}

time_t
parse_configuration(const char *filename) {
	FILE *config_stream;
	struct stat sb;

	config_stream = fopen(filename, "r");
	if (config_stream == NULL)
		ns_panic(ns_log_parser, 0, "can't open '%s'", filename);
	if (fstat(fileno(config_stream), &sb) == -1)
		ns_panic(ns_log_parser, 0, "can't stat '%s'", filename);

	lexer_setup();
	parser_setup();
	lexer_begin_file(filename, config_stream);
	(void)yyparse();
	lexer_end_file();
	parser_cleanup();
	return (sb.st_mtime);
}

void
parser_initialize(void) {
	lexer_initialize();
}

void
parser_shutdown(void) {
	if (authtab != NULL)
		free_symbol_table(authtab);
	if (channeltab != NULL)
		free_symbol_table(channeltab);
	lexer_shutdown();
}
#line 1284 "y.tab.c"
/* allocate initial stack or double stack size, up to YYMAXDEPTH */
static int yygrowstack()
{
    int newsize, i;
    short *newss;
    YYSTYPE *newvs;

    if ((newsize = yystacksize) == 0)
        newsize = YYINITSTACKSIZE;
    else if (newsize >= YYMAXDEPTH)
        return -1;
    else if ((newsize *= 2) > YYMAXDEPTH)
        newsize = YYMAXDEPTH;
    i = yyssp - yyss;
    newss = yyss ? (short *)realloc(yyss, newsize * sizeof *newss) :
      (short *)malloc(newsize * sizeof *newss);
    if (newss == NULL)
        return -1;
    yyss = newss;
    yyssp = newss + i;
    newvs = yyvs ? (YYSTYPE *)realloc(yyvs, newsize * sizeof *newvs) :
      (YYSTYPE *)malloc(newsize * sizeof *newvs);
    if (newvs == NULL)
        return -1;
    yyvs = newvs;
    yyvsp = newvs + i;
    yystacksize = newsize;
    yysslim = yyss + newsize - 1;
    return 0;
}

#define YYABORT goto yyabort
#define YYREJECT goto yyabort
#define YYACCEPT goto yyaccept
#define YYERROR goto yyerrlab

#ifndef YYPARSE_PARAM
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG void
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG
#define YYPARSE_PARAM_DECL
#endif	/* ANSI-C/C++ */
#else	/* YYPARSE_PARAM */
#ifndef YYPARSE_PARAM_TYPE
#define YYPARSE_PARAM_TYPE void *
#endif
#if defined(__cplusplus) || __STDC__
#define YYPARSE_PARAM_ARG YYPARSE_PARAM_TYPE YYPARSE_PARAM
#define YYPARSE_PARAM_DECL
#else	/* ! ANSI-C/C++ */
#define YYPARSE_PARAM_ARG YYPARSE_PARAM
#define YYPARSE_PARAM_DECL YYPARSE_PARAM_TYPE YYPARSE_PARAM;
#endif	/* ANSI-C/C++ */
#endif	/* ! YYPARSE_PARAM */

int
yyparse (YYPARSE_PARAM_ARG)
    YYPARSE_PARAM_DECL
{
    register int yym, yyn, yystate;
#if YYDEBUG
    register const char *yys;

    if ((yys = getenv("YYDEBUG")))
    {
        yyn = *yys;
        if (yyn >= '0' && yyn <= '9')
            yydebug = yyn - '0';
    }
#endif

    yynerrs = 0;
    yyerrflag = 0;
    yychar = (-1);

    if (yyss == NULL && yygrowstack()) goto yyoverflow;
    yyssp = yyss;
    yyvsp = yyvs;
    *yyssp = yystate = 0;

yyloop:
    if ((yyn = yydefred[yystate])) goto yyreduce;
    if (yychar < 0)
    {
        if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, reading %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
    }
    if ((yyn = yysindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: state %d, shifting to state %d\n",
                    YYPREFIX, yystate, yytable[yyn]);
#endif
        if (yyssp >= yysslim && yygrowstack())
        {
            goto yyoverflow;
        }
        *++yyssp = yystate = yytable[yyn];
        *++yyvsp = yylval;
        yychar = (-1);
        if (yyerrflag > 0)  --yyerrflag;
        goto yyloop;
    }
    if ((yyn = yyrindex[yystate]) && (yyn += yychar) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yychar)
    {
        yyn = yytable[yyn];
        goto yyreduce;
    }
    if (yyerrflag) goto yyinrecovery;
#if defined(lint) || defined(__GNUC__)
    goto yynewerror;
#endif
yynewerror:
    yyerror("syntax error");
#if defined(lint) || defined(__GNUC__)
    goto yyerrlab;
#endif
yyerrlab:
    ++yynerrs;
yyinrecovery:
    if (yyerrflag < 3)
    {
        yyerrflag = 3;
        for (;;)
        {
            if ((yyn = yysindex[*yyssp]) && (yyn += YYERRCODE) >= 0 &&
                    yyn <= YYTABLESIZE && yycheck[yyn] == YYERRCODE)
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: state %d, error recovery shifting\
 to state %d\n", YYPREFIX, *yyssp, yytable[yyn]);
#endif
                if (yyssp >= yysslim && yygrowstack())
                {
                    goto yyoverflow;
                }
                *++yyssp = yystate = yytable[yyn];
                *++yyvsp = yylval;
                goto yyloop;
            }
            else
            {
#if YYDEBUG
                if (yydebug)
                    printf("%sdebug: error recovery discarding state %d\n",
                            YYPREFIX, *yyssp);
#endif
                if (yyssp <= yyss) goto yyabort;
                --yyssp;
                --yyvsp;
            }
        }
    }
    else
    {
        if (yychar == 0) goto yyabort;
#if YYDEBUG
        if (yydebug)
        {
            yys = 0;
            if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
            if (!yys) yys = "illegal-symbol";
            printf("%sdebug: state %d, error recovery discards token %d (%s)\n",
                    YYPREFIX, yystate, yychar, yys);
        }
#endif
        yychar = (-1);
        goto yyloop;
    }
yyreduce:
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: state %d, reducing by rule %d (%s)\n",
                YYPREFIX, yystate, yyn, yyrule[yyn]);
#endif
    yym = yylen[yyn];
    yyval = yyvsp[1-yym];
    switch (yyn)
    {
case 1:
#line 248 "ns_parser.y"
{
		if (EMPTY(current_controls))
			ns_ctl_defaults(&current_controls);
		ns_ctl_install(&current_controls);
	}
break;
case 16:
#line 274 "ns_parser.y"
{
		lexer_begin_file(yyvsp[-1].cp, NULL);
		(void)freestr(yyvsp[-1].cp);
	}
break;
case 17:
#line 285 "ns_parser.y"
{
		if (seen_options)
			parser_error(0, "cannot redefine options");
		current_options = new_options();
	}
break;
case 18:
#line 291 "ns_parser.y"
{
		if (!seen_options)
			set_options(current_options, 0);
		else
			free_options(current_options);
		current_options = NULL;
		seen_options = 1;
	}
break;
case 22:
#line 307 "ns_parser.y"
{
		if (current_options->hostname != NULL)
			(void)freestr(current_options->hostname);
		current_options->hostname = yyvsp[0].cp;
	}
break;
case 23:
#line 313 "ns_parser.y"
{
		if (current_options->version != NULL)
			(void)freestr(current_options->version);
		current_options->version = yyvsp[0].cp;
	}
break;
case 24:
#line 319 "ns_parser.y"
{
		if (current_options->directory != NULL)
			(void)freestr(current_options->directory);
		current_options->directory = yyvsp[0].cp;
	}
break;
case 25:
#line 325 "ns_parser.y"
{
		if (current_options->named_xfer != NULL)
			(void)freestr(current_options->named_xfer);
		current_options->named_xfer = yyvsp[0].cp;
	}
break;
case 26:
#line 331 "ns_parser.y"
{
		if (current_options->pid_filename != NULL)
			(void)freestr(current_options->pid_filename);
		current_options->pid_filename = yyvsp[0].cp;
	}
break;
case 27:
#line 337 "ns_parser.y"
{
		if (current_options->stats_filename != NULL)
			(void)freestr(current_options->stats_filename);
		current_options->stats_filename = yyvsp[0].cp;
	}
break;
case 28:
#line 343 "ns_parser.y"
{
		if (current_options->memstats_filename != NULL)
			(void)freestr(current_options->memstats_filename);
		current_options->memstats_filename = yyvsp[0].cp;
	}
break;
case 29:
#line 349 "ns_parser.y"
{
		if (current_options->dump_filename != NULL)
			(void)freestr(current_options->dump_filename);
		current_options->dump_filename = yyvsp[0].cp;
	}
break;
case 30:
#line 355 "ns_parser.y"
{
		current_options->preferred_glue =
			strcasecmp(yyvsp[0].cp, "aaaa") ? T_A : T_AAAA;
	}
break;
case 31:
#line 360 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FAKE_IQUERY, yyvsp[0].num);
	}
break;
case 32:
#line 365 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_NORECURSE, !yyvsp[0].num);
	}
break;
case 33:
#line 370 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_NOFETCHGLUE, !yyvsp[0].num);
	}
break;
case 34:
#line 375 "ns_parser.y"
{
		set_global_boolean_option(current_options, 
			OPTION_HITCOUNT, yyvsp[0].num);
	}
break;
case 35:
#line 380 "ns_parser.y"
{
		current_options->notify = notify_explicit;
	}
break;
case 36:
#line 384 "ns_parser.y"
{
		if (yyvsp[0].num)
			current_options->notify = notify_yes;
		else
			current_options->notify = notify_no;
	}
break;
case 37:
#line 391 "ns_parser.y"
{
		if (initial_configuration && yyvsp[0].num)
			ns_notice(ns_log_default,
				  "suppressing initial notifies");
		set_global_boolean_option(current_options, 
			OPTION_SUPNOTIFY_INITIAL, yyvsp[0].num);
	}
break;
case 38:
#line 399 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_HOSTSTATS, yyvsp[0].num);
	}
break;
case 39:
#line 404 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_DEALLOC_ON_EXIT, yyvsp[0].num);
	}
break;
case 40:
#line 409 "ns_parser.y"
{
		set_global_boolean_option(current_options, OPTION_USE_IXFR, yyvsp[0].num);
	}
break;
case 41:
#line 413 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_MAINTAIN_IXFR_BASE, yyvsp[0].num);
	}
break;
case 42:
#line 418 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_NORFC2308_TYPE1, yyvsp[0].num);
		set_global_boolean_option(current_options,
					  OPTION_NONAUTH_NXDOMAIN, !yyvsp[0].num);
	}
break;
case 43:
#line 425 "ns_parser.y"
{
		set_global_boolean_option(current_options, OPTION_NONAUTH_NXDOMAIN,
				   !yyvsp[0].num);
	}
break;
case 44:
#line 430 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_MULTIPLE_CNAMES, yyvsp[0].num);
	}
break;
case 45:
#line 435 "ns_parser.y"
{
		current_options->check_names[yyvsp[-1].s_int] = (enum severity)yyvsp[0].s_int;
	}
break;
case 46:
#line 439 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_USE_ID_POOL, yyvsp[0].num);
	}
break;
case 47:
#line 444 "ns_parser.y"
{
		set_global_boolean_option(current_options,
                        		  OPTION_NORFC2308_TYPE1, !yyvsp[0].num);
	}
break;
case 48:
#line 449 "ns_parser.y"
{
		char port_string[10];
		symbol_value value;

		(void)sprintf(port_string, "%u", yyvsp[-3].us_int);
		if (lookup_symbol(symtab, port_string, SYM_PORT, NULL))
			parser_error(0,
				     "cannot redefine listen-on for port %u",
				     ntohs(yyvsp[-3].us_int));
		else {
			add_listen_on(current_options, yyvsp[-3].us_int, yyvsp[-1].iml);
			value.pointer = NULL;
			define_symbol(symtab, port_string, SYM_PORT, value, 0);
		}

	}
break;
case 50:
#line 467 "ns_parser.y"
{
		if (current_options->fwdtab) {
			free_forwarders(current_options->fwdtab);
			current_options->fwdtab = NULL;
		}
	}
break;
case 53:
#line 476 "ns_parser.y"
{
		current_options->axfr_src = yyvsp[0].ip_addr;
	}
break;
case 54:
#line 480 "ns_parser.y"
{
		if (current_options->query_acl) {
			parser_warning(0,
			      "options allow-query acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else 
			current_options->query_acl = yyvsp[-1].iml;
	}
break;
case 55:
#line 489 "ns_parser.y"
{
		if (current_options->recursion_acl) {
			parser_warning(0,
			      "options allow-recursion acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->recursion_acl = yyvsp[-1].iml;
	}
break;
case 56:
#line 498 "ns_parser.y"
{
		if (current_options->transfer_acl) {
			parser_warning(0,
			   "options allow-transfer acl already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else 
			current_options->transfer_acl = yyvsp[-1].iml;
	}
break;
case 57:
#line 507 "ns_parser.y"
{
		if (current_options->sortlist) {
			parser_warning(0,
			      "options sortlist already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->sortlist = yyvsp[-1].iml;
	}
break;
case 58:
#line 516 "ns_parser.y"
{
		if (current_options->also_notify) {
			parser_warning(0,
			    "duplicate also-notify clause: overwriting");
			free_also_notify(current_options);
			current_options->also_notify = NULL;
		}
	}
break;
case 60:
#line 526 "ns_parser.y"
{
		if (current_options->blackhole_acl) {
			parser_warning(0,
			      "options blackhole already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->blackhole_acl = yyvsp[-1].iml;
	}
break;
case 61:
#line 535 "ns_parser.y"
{
		if (current_options->topology) {
			parser_warning(0,
			      "options topology already set; skipping");
			free_ip_match_list(yyvsp[-1].iml);
		} else
			current_options->topology = yyvsp[-1].iml;
	}
break;
case 62:
#line 544 "ns_parser.y"
{
		/* To get around the $$ = $1 default rule. */
	}
break;
case 64:
#line 549 "ns_parser.y"
{
		current_options->transfer_format = yyvsp[0].axfr_fmt;
	}
break;
case 65:
#line 553 "ns_parser.y"
{
		current_options->max_transfer_time_in = yyvsp[0].num * 60;
	}
break;
case 66:
#line 557 "ns_parser.y"
{
		current_options->serial_queries = yyvsp[0].num;
	}
break;
case 67:
#line 561 "ns_parser.y"
{
		current_options->clean_interval = yyvsp[0].num * 60;
	}
break;
case 68:
#line 565 "ns_parser.y"
{
		current_options->interface_interval = yyvsp[0].num * 60;
	}
break;
case 69:
#line 569 "ns_parser.y"
{
		current_options->stats_interval = yyvsp[0].num * 60;
	}
break;
case 70:
#line 573 "ns_parser.y"
{
		current_options->max_host_stats = yyvsp[0].num;
	}
break;
case 71:
#line 577 "ns_parser.y"
{
		current_options->max_log_size_ixfr = yyvsp[0].ul_int;
	}
break;
case 72:
#line 581 "ns_parser.y"
{
		current_options->max_ncache_ttl = yyvsp[0].num;
	}
break;
case 73:
#line 585 "ns_parser.y"
{
		current_options->lame_ttl = yyvsp[0].num;
	}
break;
case 74:
#line 589 "ns_parser.y"
{
		current_options->heartbeat_interval = yyvsp[0].num * 60;
	}
break;
case 75:
#line 593 "ns_parser.y"
{
		set_global_boolean_option(current_options,
                                          OPTION_NODIALUP, !yyvsp[0].num);
	}
break;
case 76:
#line 598 "ns_parser.y"
{
		if (current_options->ordering)
			free_rrset_order_list(current_options->ordering);
		current_options->ordering = yyvsp[-1].rol;
	}
break;
case 77:
#line 604 "ns_parser.y"
{
		set_global_boolean_option(current_options,
					  OPTION_TREAT_CR_AS_SPACE, yyvsp[0].num);
	}
break;
case 78:
#line 609 "ns_parser.y"
{
		if (yyvsp[0].num >= 1)
			current_options->minroots = yyvsp[0].num;
	}
break;
case 80:
#line 615 "ns_parser.y"
{
		if (yyvsp[0].num < 512)
			current_options->edns_udp_size = 512;
		else if (yyvsp[0].num > EDNS_MESSAGE_SZ)
			current_options->edns_udp_size = EDNS_MESSAGE_SZ;
		else
			current_options->edns_udp_size = yyvsp[0].num;
	}
break;
case 86:
#line 639 "ns_parser.y"
{
		ns_ctl_add(&current_controls, ns_ctl_new_inet(yyvsp[-6].ip_addr, yyvsp[-4].us_int, yyvsp[-1].iml));
	}
break;
case 87:
#line 645 "ns_parser.y"
{
		parser_warning(0, "Ignoring BIND 9 inet control clause");
		free_ip_match_list(yyvsp[-5].iml);
	}
break;
case 88:
#line 652 "ns_parser.y"
{
		parser_warning(0, "Ignoring BIND 9 inet control clause");
		free_ip_match_list(yyvsp[-5].iml);
	}
break;
case 89:
#line 657 "ns_parser.y"
{
#ifndef NO_SOCKADDR_UN
		ns_ctl_add(&current_controls, ns_ctl_new_unix(yyvsp[-6].cp, yyvsp[-4].num, yyvsp[-2].num, yyvsp[0].num));
#endif
		freestr(yyvsp[-6].cp);
	}
break;
case 91:
#line 667 "ns_parser.y"
{
		rrset_order_list rol;

		rol = new_rrset_order_list();
		if (yyvsp[-1].roe != NULL) {
			add_to_rrset_order_list(rol, yyvsp[-1].roe);
		}
		
		yyval.rol = rol;
	}
break;
case 92:
#line 678 "ns_parser.y"
{
		if (yyvsp[-1].roe != NULL) {
			add_to_rrset_order_list(yyvsp[-2].rol, yyvsp[-1].roe);
		}
		yyval.rol = yyvsp[-2].rol;
	}
break;
case 93:
#line 687 "ns_parser.y"
{
		yyval.s_int = C_ANY;
	}
break;
case 94:
#line 691 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CLASS, &value))
			yyval.s_int = value.integer;
		else {
			parser_error(0, "unknown class '%s'; using ANY", yyvsp[0].cp);
			yyval.s_int = C_ANY;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 95:
#line 705 "ns_parser.y"
{
		yyval.s_int = ns_t_any;
	}
break;
case 96:
#line 709 "ns_parser.y"
{
		int success;

		if (strcmp(yyvsp[0].cp, "*") == 0) {
			yyval.s_int = ns_t_any;
		} else {
			yyval.s_int = __sym_ston(__p_type_syms, yyvsp[0].cp, &success);
			if (success == 0) {
				yyval.s_int = ns_t_any;
				parser_error(0,
					     "unknown type '%s'; assuming ANY",
					     yyvsp[0].cp);
			}
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 97:
#line 728 "ns_parser.y"
{
		yyval.cp = savestr("*", 1);
	}
break;
case 98:
#line 732 "ns_parser.y"
{
		if (strcmp(".",yyvsp[0].cp) == 0 || strcmp("*.",yyvsp[0].cp) == 0) {
			yyval.cp = savestr("*", 1);
			(void)freestr(yyvsp[0].cp);
		} else {
			yyval.cp = yyvsp[0].cp ;
		}
		/* XXX Should do any more name validation here? */
	}
break;
case 99:
#line 744 "ns_parser.y"
{
		enum ordering o;

		if (strlen(yyvsp[0].cp) == 0) {
			parser_error(0, "null order name");
			yyval.roe = NULL ;
		} else {
			o = lookup_ordering(yyvsp[0].cp);
			if (o == unknown_order) {
				o = (enum ordering)DEFAULT_ORDERING;
				parser_error(0,
					     "invalid order name '%s'; using %s",
					     yyvsp[0].cp, p_order(o));
			}
			
			(void)freestr(yyvsp[0].cp);
			
			yyval.roe = new_rrset_order_element(yyvsp[-4].s_int, yyvsp[-3].s_int, yyvsp[-2].cp, o);
		}
	}
break;
case 100:
#line 767 "ns_parser.y"
{
		yyval.axfr_fmt = axfr_one_answer;
	}
break;
case 101:
#line 771 "ns_parser.y"
{
		yyval.axfr_fmt = axfr_many_answers;
	}
break;
case 102:
#line 776 "ns_parser.y"
{ yyval.ip_addr = yyvsp[0].ip_addr; }
break;
case 103:
#line 777 "ns_parser.y"
{ yyval.ip_addr.s_addr = htonl(INADDR_ANY); }
break;
case 104:
#line 780 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 105:
#line 781 "ns_parser.y"
{ yyval.us_int = htons(0); }
break;
case 106:
#line 785 "ns_parser.y"
{
		current_options->query_source.sin_addr = yyvsp[0].ip_addr;
	}
break;
case 107:
#line 791 "ns_parser.y"
{
		current_options->query_source.sin_port = yyvsp[0].us_int;
	}
break;
case 112:
#line 802 "ns_parser.y"
{ yyval.us_int = htons(NS_DEFAULTPORT); }
break;
case 113:
#line 803 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 114:
#line 806 "ns_parser.y"
{ yyval.us_int = htons(0); }
break;
case 115:
#line 807 "ns_parser.y"
{ yyval.us_int = yyvsp[0].us_int; }
break;
case 116:
#line 812 "ns_parser.y"
{ 
		yyval.num = 1;	
	}
break;
case 117:
#line 816 "ns_parser.y"
{ 
		yyval.num = 1;	
	}
break;
case 118:
#line 820 "ns_parser.y"
{ 
		yyval.num = 0;	
	}
break;
case 119:
#line 824 "ns_parser.y"
{ 
		yyval.num = 0;	
	}
break;
case 120:
#line 828 "ns_parser.y"
{ 
		if (yyvsp[0].num == 1 || yyvsp[0].num == 0) {
			yyval.num = yyvsp[0].num;
		} else {
			parser_warning(0,
				       "number should be 0 or 1; assuming 1");
			yyval.num = 1;
		}
	}
break;
case 121:
#line 840 "ns_parser.y"
{
		yyval.s_int = primary_trans;
	}
break;
case 122:
#line 844 "ns_parser.y"
{
		yyval.s_int = secondary_trans;
	}
break;
case 123:
#line 848 "ns_parser.y"
{
		yyval.s_int = response_trans;
	}
break;
case 124:
#line 854 "ns_parser.y"
{
		yyval.s_int = warn;
	}
break;
case 125:
#line 858 "ns_parser.y"
{
		yyval.s_int = fail;
	}
break;
case 126:
#line 862 "ns_parser.y"
{
		yyval.s_int = ignore;
	}
break;
case 127:
#line 868 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FORWARD_ONLY, 1);
	}
break;
case 128:
#line 873 "ns_parser.y"
{
		set_global_boolean_option(current_options,
			OPTION_FORWARD_ONLY, 0);
	}
break;
case 129:
#line 878 "ns_parser.y"
{
		parser_warning(0, "forward if-no-answer is unimplemented");
	}
break;
case 130:
#line 882 "ns_parser.y"
{
		parser_warning(0, "forward if-no-domain is unimplemented");
	}
break;
case 131:
#line 888 "ns_parser.y"
{
		current_options->data_size = yyvsp[0].ul_int;
	}
break;
case 132:
#line 892 "ns_parser.y"
{
		current_options->stack_size = yyvsp[0].ul_int;
	}
break;
case 133:
#line 896 "ns_parser.y"
{
		current_options->core_size = yyvsp[0].ul_int;
	}
break;
case 134:
#line 900 "ns_parser.y"
{
		current_options->files = yyvsp[0].ul_int;
	}
break;
case 135:
#line 906 "ns_parser.y"
{
		u_long result;

		if (unit_to_ulong(yyvsp[0].cp, &result))
			yyval.ul_int = result;
		else {
			parser_error(0, "invalid unit string '%s'", yyvsp[0].cp);
			/* 0 means "use default" */
			yyval.ul_int = 0;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 136:
#line 919 "ns_parser.y"
{	
		yyval.ul_int = (u_long)yyvsp[0].num;
	}
break;
case 137:
#line 923 "ns_parser.y"
{
		yyval.ul_int = 0;
	}
break;
case 138:
#line 927 "ns_parser.y"
{
		yyval.ul_int = ULONG_MAX;
	}
break;
case 139:
#line 933 "ns_parser.y"
{
		current_options->transfers_in = (u_long) yyvsp[0].num;
	}
break;
case 140:
#line 937 "ns_parser.y"
{
		current_options->transfers_out = (u_long) yyvsp[0].num;
	}
break;
case 141:
#line 941 "ns_parser.y"
{
		current_options->transfers_per_ns = (u_long) yyvsp[0].num;
	}
break;
case 144:
#line 951 "ns_parser.y"
{
		/* nothing */
	}
break;
case 145:
#line 955 "ns_parser.y"
{
		/* nothing */
	}
break;
case 146:
#line 961 "ns_parser.y"
{
	  	add_global_forwarder(current_options, yyvsp[0].ip_addr);
	}
break;
case 149:
#line 971 "ns_parser.y"
{
		/* nothing */
	}
break;
case 150:
#line 975 "ns_parser.y"
{
		/* nothing */
	}
break;
case 151:
#line 981 "ns_parser.y"
{
	  	add_global_also_notify(current_options, yyvsp[0].ip_addr);
	}
break;
case 152:
#line 991 "ns_parser.y"
{
		current_logging = begin_logging();
	}
break;
case 153:
#line 995 "ns_parser.y"
{
		end_logging(current_logging, 1);
		current_logging = NULL;
	}
break;
case 157:
#line 1007 "ns_parser.y"
{
		current_category = yyvsp[0].s_int;
	}
break;
case 159:
#line 1012 "ns_parser.y"
{
		chan_type = log_null;
		chan_flags = 0;
		chan_level = log_info;
	}
break;
case 160:
#line 1018 "ns_parser.y"
{
		log_channel current_channel = NULL;

		if (lookup_channel(yyvsp[-4].cp) != NULL) {
			parser_error(0, "can't redefine channel '%s'", yyvsp[-4].cp);
		} else {
			switch (chan_type) {
			case log_file:
				current_channel =
					log_new_file_channel(chan_flags,
							     chan_level,
							     chan_name, NULL,
							     chan_versions,
							     chan_max_size);
				log_set_file_owner(current_channel,
						   user_id, group_id);
				chan_name = freestr(chan_name);
				break;
			case log_syslog:
				current_channel =
					log_new_syslog_channel(chan_flags,
							       chan_level,
							       chan_facility);
				break;
			case log_null:
				current_channel = log_new_null_channel();
				break;
			default:
				ns_panic(ns_log_parser, 1,
					 "unknown channel type: %d",
					 chan_type);
			}
			if (current_channel == NULL)
				ns_panic(ns_log_parser, 0,
					 "couldn't create channel");
			define_channel(yyvsp[-4].cp, current_channel);
		}
		(void)freestr(yyvsp[-4].cp);
	}
break;
case 161:
#line 1060 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_LOGGING, &value)) {
			chan_level = value.integer;
		} else {
			parser_error(0, "unknown severity '%s'", yyvsp[0].cp);
			chan_level = log_debug(99);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 162:
#line 1072 "ns_parser.y"
{
		chan_level = log_debug(1);
	}
break;
case 163:
#line 1076 "ns_parser.y"
{
		chan_level = yyvsp[0].num;
	}
break;
case 164:
#line 1080 "ns_parser.y"
{
		chan_level = 0;
		chan_flags |= LOG_USE_CONTEXT_LEVEL|LOG_REQUIRE_DEBUG;
	}
break;
case 165:
#line 1087 "ns_parser.y"
{
		chan_versions = yyvsp[0].num;
	}
break;
case 166:
#line 1091 "ns_parser.y"
{
		chan_versions = LOG_MAX_VERSIONS;
	}
break;
case 167:
#line 1097 "ns_parser.y"
{
		chan_max_size = yyvsp[0].ul_int;
	}
break;
case 168:
#line 1103 "ns_parser.y"
{
		chan_versions = 0;
		chan_max_size = ULONG_MAX;
	}
break;
case 169:
#line 1108 "ns_parser.y"
{
		chan_max_size = ULONG_MAX;
	}
break;
case 170:
#line 1112 "ns_parser.y"
{
		chan_versions = 0;
	}
break;
case 173:
#line 1120 "ns_parser.y"
{
		chan_flags |= LOG_CLOSE_STREAM;
		chan_type = log_file;
		chan_name = yyvsp[-1].cp;
	}
break;
case 174:
#line 1128 "ns_parser.y"
{ yyval.cp = yyvsp[0].cp; }
break;
case 175:
#line 1129 "ns_parser.y"
{ yyval.cp = savestr("syslog", 1); }
break;
case 176:
#line 1132 "ns_parser.y"
{ yyval.s_int = LOG_DAEMON; }
break;
case 177:
#line 1134 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_SYSLOG, &value)) {
			yyval.s_int = value.integer;
		} else {
			parser_error(0, "unknown facility '%s'", yyvsp[0].cp);
			yyval.s_int = LOG_DAEMON;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 178:
#line 1148 "ns_parser.y"
{
		chan_type = log_syslog;
		chan_facility = yyvsp[0].s_int;
	}
break;
case 179:
#line 1154 "ns_parser.y"
{ /* nothing to do */ }
break;
case 180:
#line 1155 "ns_parser.y"
{ /* nothing to do */ }
break;
case 181:
#line 1157 "ns_parser.y"
{
		chan_type = log_null;
	}
break;
case 182:
#line 1160 "ns_parser.y"
{ /* nothing to do */ }
break;
case 183:
#line 1162 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_TIMESTAMP;
		else
			chan_flags &= ~LOG_TIMESTAMP;
	}
break;
case 184:
#line 1169 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_PRINT_CATEGORY;
		else
			chan_flags &= ~LOG_PRINT_CATEGORY;
	}
break;
case 185:
#line 1176 "ns_parser.y"
{
		if (yyvsp[0].num)
			chan_flags |= LOG_PRINT_LEVEL;
		else
			chan_flags &= ~LOG_PRINT_LEVEL;
	}
break;
case 190:
#line 1190 "ns_parser.y"
{ yyval.cp = savestr("null", 1); }
break;
case 191:
#line 1194 "ns_parser.y"
{
		log_channel channel;

		if (current_category >= 0) {
			channel = lookup_channel(yyvsp[0].cp);
			if (channel != NULL) {
				add_log_channel(current_logging,
						current_category, channel);
			} else
				parser_error(0, "unknown channel '%s'", yyvsp[0].cp);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 196:
#line 1215 "ns_parser.y"
{ yyval.cp = savestr("default", 1); }
break;
case 197:
#line 1216 "ns_parser.y"
{ yyval.cp = savestr("notify", 1); }
break;
case 198:
#line 1220 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CATEGORY, &value))
			yyval.s_int = value.integer;
		else {
			parser_error(0, "invalid logging category '%s'",
				     yyvsp[0].cp);
			yyval.s_int = -1;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 199:
#line 1239 "ns_parser.y"
{
		const char *ip_printable;
		symbol_value value;
		
		ip_printable = inet_ntoa(yyvsp[0].ip_addr);
		value.pointer = NULL;
		if (lookup_symbol(symtab, ip_printable, SYM_SERVER, NULL))
			seen_server = 1;
		else
			seen_server = 0;
		if (seen_server)
			parser_error(0, "cannot redefine server '%s'", 
				     ip_printable);
		else
			define_symbol(symtab, ip_printable, SYM_SERVER, value,
				      0);
		current_server = begin_server(yyvsp[0].ip_addr);
	}
break;
case 200:
#line 1258 "ns_parser.y"
{
		end_server(current_server, !seen_server);
	}
break;
case 203:
#line 1268 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_BOGUS, yyvsp[0].num);
	}
break;
case 204:
#line 1272 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_SUPPORT_IXFR, yyvsp[0].num);
	}
break;
case 205:
#line 1276 "ns_parser.y"
{
		set_server_transfers(current_server, (int)yyvsp[0].num);
	}
break;
case 206:
#line 1280 "ns_parser.y"
{
		set_server_transfer_format(current_server, yyvsp[0].axfr_fmt);
	}
break;
case 208:
#line 1285 "ns_parser.y"
{
		set_server_option(current_server, SERVER_INFO_EDNS, yyvsp[0].num);
	}
break;
case 210:
#line 1296 "ns_parser.y"
{
		ip_match_list iml;
		
		iml = new_ip_match_list();
		if (yyvsp[-1].ime != NULL)
			add_to_ip_match_list(iml, yyvsp[-1].ime);
		yyval.iml = iml;
	}
break;
case 211:
#line 1305 "ns_parser.y"
{
		if (yyvsp[-1].ime != NULL)
			add_to_ip_match_list(yyvsp[-2].iml, yyvsp[-1].ime);
		yyval.iml = yyvsp[-2].iml;
	}
break;
case 213:
#line 1314 "ns_parser.y"
{
		if (yyvsp[0].ime != NULL)
			ip_match_negate(yyvsp[0].ime);
		yyval.ime = yyvsp[0].ime;
	}
break;
case 214:
#line 1320 "ns_parser.y"
{
		char *key_name;
		struct dst_key *dst_key;

		key_name = canonical_name(yyvsp[0].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[0].cp);
			key_name = savestr("__bad_key__", 1);
		}
		dst_key = find_key(key_name, NULL);
		if (dst_key == NULL) {
			parser_error(0, "key \"%s\" not found", key_name);
			yyval.ime = NULL;
		}
		else
			yyval.ime = new_ip_match_key(dst_key);
	        (void)freestr(key_name);
		freestr(yyvsp[0].cp);
	}
break;
case 215:
#line 1343 "ns_parser.y"
{
		yyval.ime = new_ip_match_pattern(yyvsp[0].ip_addr, 32);
	}
break;
case 216:
#line 1347 "ns_parser.y"
{
		if (yyvsp[0].num < 0 || yyvsp[0].num > 32) {
			parser_error(0, "mask bits out of range; skipping");
			yyval.ime = NULL;
		} else {
			yyval.ime = new_ip_match_pattern(yyvsp[-2].ip_addr, yyvsp[0].num);
			if (yyval.ime == NULL)
				parser_error(0, 
					   "address/mask mismatch; skipping");
		}
	}
break;
case 217:
#line 1359 "ns_parser.y"
{
		struct in_addr ia;

		if (yyvsp[-2].num > 255) {
			parser_error(0, "address out of range; skipping");
			yyval.ime = NULL;
		} else {
			if (yyvsp[0].num < 0 || yyvsp[0].num > 32) {
				parser_error(0,
					"mask bits out of range; skipping");
					yyval.ime = NULL;
			} else {
				ia.s_addr = htonl((yyvsp[-2].num & 0xff) << 24);
				yyval.ime = new_ip_match_pattern(ia, yyvsp[0].num);
				if (yyval.ime == NULL)
					parser_error(0, 
					   "address/mask mismatch; skipping");
			}
		}
	}
break;
case 219:
#line 1381 "ns_parser.y"
{
		char name[256];

		/*
		 * We want to be able to clean up this iml later so
		 * we give it a name and treat it like any other acl.
		 */
		sprintf(name, "__internal_%p", yyvsp[-1].iml);
		define_acl(name, yyvsp[-1].iml);
  		yyval.ime = new_ip_match_indirect(yyvsp[-1].iml);
	}
break;
case 220:
#line 1395 "ns_parser.y"
{
		ip_match_list iml;

		iml = lookup_acl(yyvsp[0].cp);
		if (iml == NULL) {
			parser_error(0, "unknown ACL '%s'", yyvsp[0].cp);
			yyval.ime = NULL;
		} else
			yyval.ime = new_ip_match_indirect(iml);
		(void)freestr(yyvsp[0].cp);
	}
break;
case 221:
#line 1413 "ns_parser.y"
{
		struct dst_key *dst_key;
		char *key_name;

		key_name = canonical_name(yyvsp[0].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[0].cp);
			yyval.keyi = NULL;
		} else {
			dst_key = lookup_key(key_name);
			if (dst_key == NULL) {
				parser_error(0, "unknown key '%s'", key_name);
				yyval.keyi = NULL;
			} else
				yyval.keyi = dst_key;
			key_name = freestr(key_name);
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 222:
#line 1436 "ns_parser.y"
{
		if (yyvsp[0].keyi == NULL)
			parser_error(0, "empty key not added to server list ");
		else
			add_server_key_info(current_server, yyvsp[0].keyi);
	}
break;
case 226:
#line 1449 "ns_parser.y"
{ /* empty */ }
break;
case 230:
#line 1457 "ns_parser.y"
{
		current_algorithm = NULL;
		current_secret = NULL;
	}
break;
case 231:
#line 1462 "ns_parser.y"
{
		struct dst_key *dst_key;
		char *key_name;

		key_name = canonical_name(yyvsp[-3].cp);
		if (key_name == NULL) {
			parser_error(0, "can't make key name '%s' canonical",
				     yyvsp[-3].cp);
		} else if (lookup_key(key_name) != NULL) {
			parser_error(0, "can't redefine key '%s'", key_name);
		} else {
			if (current_algorithm == NULL ||
			    current_secret == NULL)  {
				parser_error(0, "skipping bad key '%s'",
					     key_name);
			} else {
				dst_key = new_key_info(key_name,
						       current_algorithm,
						       current_secret);
				if (dst_key != NULL) {
					define_key(key_name, dst_key);
					if (secretkey_info == NULL)
						secretkey_info =
							new_key_info_list();
					add_to_key_info_list(secretkey_info,
							     dst_key);
				}
			}
		}
		if (key_name != NULL)
			key_name = freestr(key_name);
		if (current_algorithm != NULL)
			current_algorithm = freestr(current_algorithm);
		if (current_secret != NULL)
			current_secret = freestr(current_secret);
		(void)freestr(yyvsp[-3].cp);
	}
break;
case 232:
#line 1502 "ns_parser.y"
{
		current_algorithm = yyvsp[-1].cp;
		current_secret = yyvsp[0].cp;
	}
break;
case 233:
#line 1507 "ns_parser.y"
{
		current_algorithm = yyvsp[0].cp;
		current_secret = yyvsp[-1].cp;
	}
break;
case 234:
#line 1512 "ns_parser.y"
{
		current_algorithm = NULL;
		current_secret = NULL;
	}
break;
case 235:
#line 1518 "ns_parser.y"
{ yyval.cp = yyvsp[-1].cp; }
break;
case 236:
#line 1521 "ns_parser.y"
{ yyval.cp = yyvsp[-1].cp; }
break;
case 237:
#line 1529 "ns_parser.y"
{
		if (lookup_acl(yyvsp[-3].cp) != NULL) {
			parser_error(0, "can't redefine ACL '%s'", yyvsp[-3].cp);
		} else
			define_acl(yyvsp[-3].cp, yyvsp[-1].iml);
		(void)freestr(yyvsp[-3].cp);
	}
break;
case 238:
#line 1543 "ns_parser.y"
{
		int sym_type;
		symbol_value value;
		char *zone_name;

		if (!seen_options && !logged_options_error) {
			parser_error(0,
             "no options statement before first zone; using previous/default");
			logged_options_error = 1;
		}
		sym_type = SYM_ZONE | (yyvsp[0].num & 0xffff);
		value.pointer = NULL;
		zone_name = canonical_name(yyvsp[-1].cp);
		if (zone_name == NULL) {
			parser_error(0, "can't make zone name '%s' canonical",
				     yyvsp[-1].cp);
			should_install = 0;
			zone_name = savestr("__bad_zone__", 1);
		} else {
			if (lookup_symbol(symtab, zone_name, sym_type, NULL)) {
				should_install = 0;
				parser_error(0,
					"cannot redefine zone '%s' class %s",
					     *zone_name ? zone_name : ".",
					     p_class(yyvsp[0].num));
			} else {
				should_install = 1;
				define_symbol(symtab, zone_name, sym_type,
					      value, 0);
			}
		}
		(void)freestr(yyvsp[-1].cp);
		current_zone = begin_zone(zone_name, yyvsp[0].num); 
	}
break;
case 239:
#line 1578 "ns_parser.y"
{
		end_zone(current_zone, should_install);
	}
break;
case 242:
#line 1588 "ns_parser.y"
{
		yyval.num = C_IN;
	}
break;
case 243:
#line 1592 "ns_parser.y"
{
		symbol_value value;

		if (lookup_symbol(constants, yyvsp[0].cp, SYM_CLASS, &value))
			yyval.num = value.integer;
		else {
			/* the zone validator will give the error */
			yyval.num = C_NONE;
		}
		(void)freestr(yyvsp[0].cp);
	}
break;
case 244:
#line 1606 "ns_parser.y"
{
		yyval.s_int = Z_MASTER;
	}
break;
case 245:
#line 1610 "ns_parser.y"
{
		yyval.s_int = Z_SLAVE;
	}
break;
case 246:
#line 1614 "ns_parser.y"
{
		yyval.s_int = Z_HINT;
	}
break;
case 247:
#line 1618 "ns_parser.y"
{
		yyval.s_int = Z_STUB;
	}
break;
case 248:
#line 1622 "ns_parser.y"
{
		yyval.s_int = Z_FORWARD;
	}
break;
case 251:
#line 1632 "ns_parser.y"
{
		if (!set_zone_type(current_zone, yyvsp[0].s_int))
			parser_warning(0, "zone type already set; skipping");
	}
break;
case 252:
#line 1637 "ns_parser.y"
{
		if (!set_zone_filename(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone filename already set; skipping");
	}
break;
case 253:
#line 1643 "ns_parser.y"
{
		if (!set_zone_ixfr_file(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone ixfr data base already set; skipping");
	}
break;
case 254:
#line 1649 "ns_parser.y"
{
		if (!set_zone_ixfr_tmp(current_zone, yyvsp[0].cp))
			parser_warning(0,
				       "zone ixfr temp filename already set; skipping");
	}
break;
case 255:
#line 1655 "ns_parser.y"
{
		set_zone_master_port(current_zone, yyvsp[-3].us_int);
	}
break;
case 256:
#line 1659 "ns_parser.y"
{
		set_zone_transfer_source(current_zone, yyvsp[0].ip_addr);
	}
break;
case 257:
#line 1663 "ns_parser.y"
{
		if (!set_zone_checknames(current_zone, (enum severity)yyvsp[0].s_int))
			parser_warning(0,
	                              "zone checknames already set; skipping");
	}
break;
case 258:
#line 1669 "ns_parser.y"
{
		if (!set_zone_update_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				      "zone update acl already set; skipping");
	}
break;
case 259:
#line 1675 "ns_parser.y"
{
		if (!set_zone_query_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				      "zone query acl already set; skipping");
	}
break;
case 260:
#line 1681 "ns_parser.y"
{
		if (!set_zone_transfer_acl(current_zone, yyvsp[-1].iml))
			parser_warning(0,
				    "zone transfer acl already set; skipping");
	}
break;
case 262:
#line 1688 "ns_parser.y"
{
		struct zoneinfo *zp = current_zone.opaque;
		if (zp->z_fwdtab) {
                	free_forwarders(zp->z_fwdtab);
			zp->z_fwdtab = NULL;
		}

	}
break;
case 264:
#line 1698 "ns_parser.y"
{
		if (!set_zone_transfer_time_in(current_zone, yyvsp[0].num*60))
			parser_warning(0,
		       "zone max transfer time (in) already set; skipping");
	}
break;
case 265:
#line 1704 "ns_parser.y"
{
		set_zone_max_log_size_ixfr(current_zone, yyvsp[0].ul_int);
        }
break;
case 266:
#line 1708 "ns_parser.y"
{
		set_zone_notify(current_zone, notify_explicit);
	}
break;
case 267:
#line 1712 "ns_parser.y"
{
		if (yyvsp[0].num)
			set_zone_notify(current_zone, notify_yes);
		else
			set_zone_notify(current_zone, notify_no);
	}
break;
case 268:
#line 1719 "ns_parser.y"
{
		set_zone_maintain_ixfr_base(current_zone, yyvsp[0].num);
	}
break;
case 269:
#line 1723 "ns_parser.y"
{
		/* flags proto alg key */
		set_zone_pubkey(current_zone, yyvsp[-3].num, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 270:
#line 1728 "ns_parser.y"
{
		/* flags proto alg key */
		char *endp;
		int flags = (int) strtol(yyvsp[-3].cp, &endp, 0);
		if (*endp != '\0')
			ns_panic(ns_log_parser, 1,
				 "Invalid flags string: %s", yyvsp[-3].cp);
		set_zone_pubkey(current_zone, flags, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);

	}
break;
case 272:
#line 1740 "ns_parser.y"
{
		 set_zone_dialup(current_zone, yyvsp[0].num);
	}
break;
case 274:
#line 1747 "ns_parser.y"
{
		/* nothing */
	}
break;
case 275:
#line 1751 "ns_parser.y"
{
		/* nothing */
	}
break;
case 276:
#line 1757 "ns_parser.y"
{
	  	add_zone_master(current_zone, yyvsp[0].ip_addr, NULL);
	}
break;
case 277:
#line 1761 "ns_parser.y"
{
		add_zone_master(current_zone, yyvsp[-2].ip_addr, yyvsp[0].keyi);
	}
break;
case 280:
#line 1771 "ns_parser.y"
{
		/* nothing */
	}
break;
case 281:
#line 1775 "ns_parser.y"
{
		/* nothing */
	}
break;
case 282:
#line 1781 "ns_parser.y"
{
	  	add_zone_notify(current_zone, yyvsp[0].ip_addr);
	}
break;
case 283:
#line 1787 "ns_parser.y"
{
		set_zone_boolean_option(current_zone, OPTION_FORWARD_ONLY, 1);
	}
break;
case 284:
#line 1791 "ns_parser.y"
{
		set_zone_boolean_option(current_zone, OPTION_FORWARD_ONLY, 0);
	}
break;
case 285:
#line 1797 "ns_parser.y"
{
		set_zone_forward(current_zone);
	}
break;
case 287:
#line 1804 "ns_parser.y"
{
		/* nothing */
	}
break;
case 288:
#line 1808 "ns_parser.y"
{
		/* nothing */
	}
break;
case 289:
#line 1814 "ns_parser.y"
{
	  	add_zone_forwarder(current_zone, yyvsp[0].ip_addr);
	}
break;
case 290:
#line 1824 "ns_parser.y"
{
	}
break;
case 291:
#line 1828 "ns_parser.y"
{
		/* nothing */
	}
break;
case 292:
#line 1832 "ns_parser.y"
{
		/* nothing */
	}
break;
case 293:
#line 1837 "ns_parser.y"
{
		/* name flags proto alg key */
		set_trusted_key(yyvsp[-4].cp, yyvsp[-3].num, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 294:
#line 1842 "ns_parser.y"
{
		/* name flags proto alg key */
		char *endp;
		int flags = (int) strtol(yyvsp[-3].cp, &endp, 0);
		if (*endp != '\0')
			ns_panic(ns_log_parser, 1,
				 "Invalid flags string: %s", yyvsp[-3].cp);
		set_trusted_key(yyvsp[-4].cp, flags, yyvsp[-2].num, yyvsp[-1].num, yyvsp[0].cp);
	}
break;
case 295:
#line 1858 "ns_parser.y"
{
		if (yyvsp[0].num < 0 || yyvsp[0].num > 65535) {
		  	parser_warning(0, 
			  "invalid IP port number '%d'; setting port to 0",
			               (int)yyvsp[0].num);
			yyvsp[0].num = 0;
		} else
			yyval.us_int = htons(yyvsp[0].num);
	}
break;
#line 3266 "y.tab.c"
    }
    yyssp -= yym;
    yystate = *yyssp;
    yyvsp -= yym;
    yym = yylhs[yyn];
    if (yystate == 0 && yym == 0)
    {
#if YYDEBUG
        if (yydebug)
            printf("%sdebug: after reduction, shifting from state 0 to\
 state %d\n", YYPREFIX, YYFINAL);
#endif
        yystate = YYFINAL;
        *++yyssp = YYFINAL;
        *++yyvsp = yyval;
        if (yychar < 0)
        {
            if ((yychar = yylex()) < 0) yychar = 0;
#if YYDEBUG
            if (yydebug)
            {
                yys = 0;
                if (yychar <= YYMAXTOKEN) yys = yyname[yychar];
                if (!yys) yys = "illegal-symbol";
                printf("%sdebug: state %d, reading %d (%s)\n",
                        YYPREFIX, YYFINAL, yychar, yys);
            }
#endif
        }
        if (yychar == 0) goto yyaccept;
        goto yyloop;
    }
    if ((yyn = yygindex[yym]) && (yyn += yystate) >= 0 &&
            yyn <= YYTABLESIZE && yycheck[yyn] == yystate)
        yystate = yytable[yyn];
    else
        yystate = yydgoto[yym];
#if YYDEBUG
    if (yydebug)
        printf("%sdebug: after reduction, shifting from state %d \
to state %d\n", YYPREFIX, *yyssp, yystate);
#endif
    if (yyssp >= yysslim && yygrowstack())
    {
        goto yyoverflow;
    }
    *++yyssp = yystate;
    *++yyvsp = yyval;
    goto yyloop;
yyoverflow:
    yyerror("yacc stack overflow");
yyabort:
    return (1);
yyaccept:
    return (0);
}
